.\" set up some fancy strings
.iet .ds TX T\h'-.1667m'\v'.25m'E\v'-.25m'\h'-.125m'X
.el  .ds TX TeX
.iet .ds MF \fHMETAFONT\fP
.el  .ds MF METAFONT
.iet .ds Ps P\s-1OST\s0S\s-1CRIPT\s0
.el  .ds Ps PostScript
.TH s.medp
.SH NAME
\fIs.medp\fR \- Median polish for a GRASS sites list.
.br
.I (GRASS Sites Program)
.SH SYNOPSIS
\fBs.medp\fR
.br
\fBs.medp help\fR
.br
\fBs.medp \fR[\fB-aeqs\fR] \fBsites\*=\fIname \fBvect\*=\fIname 
\fBoutput\*=\fIname\fR [\fBreport\*=\fIname\fR] [\fBthresh\*=\fIn\fR]
.SH DESCRIPTION
.I s.medp
performs median polish on an existing sites list.
The sites are overlayed onto a grid (\fBvect\*=\fIname\fR) and each site
is associated with the closest node.  Therefore, sites do not
necessarily have to be oriented on a grid.  Each node may be
associated with zero, one, or more sites and the grid may be rotated.
Also, the length of each box does not have to be equal to width of each box
defining the grid.
.LP
Once sites are associated with nodes on a \fIp\fR by \fIq\fR grid, 
\fIp+q+1\fR extra storage locations are created (initialized to zero)
to store \fIall\fR, \fIrow\fR, and \fIcolumn\fR effects.  
The median of each row is removed from the data and added to
the extra \fIp\fR cells. Then, medians are removed from the data
as well as from the \fIp\fR cells containing \fIrow\fR effects.
The medians of data  in this pass are stored in 
the extra \fIq\fR cells (\fIcolumn\fR effects)
and the median of the row effects is stored in the extra (\fIp+1, q+1\fR)
cell (\fIall\fR effect). This is repeated until each successive
iteration leaves each site unchanged (within a tolerance, described by
[\fBthresh\*=\fIn\fR]).
.LP
\fBFlags:\fR
.IP \fB-a\fR 18
Use all sites found in the named \fIsites\fR file, rather than
limiting output to sites falling within the current geographic region.
.IP \fB-e\fR 18
Store row, column, and all effects in output file.
.IP \fB-s\fR 18
Write results to a sites list file (default is to write points in a binary 
vector file).
.IP \fB-q\fR 18
Quiet. Cut out the chatter.
.LP
\fBParameters:\fR
.IP \fBsites\*=\fIname\fR 18
Name of an existing sites file.
.IP \fBvect\*=\fIname\fR 18
Name of the grid file (binary vector file).
.IP \fBoutput\*=\fIname\fR 18
Name of the output file (binary vector file or sites list).
.IP \fBreport\*=\fIname\fR 18
Name of an ASCII file which shows original and residual data in
tabular form. 
.IP \fBthresh\*=\fIn\fR 18
Threshold to determine when convergence of median polish is obtained.
(default = 1).
.LP
\fIs.medp\fR can be run either non-interactively or interactively.
The program will be run non-interactively if the user specifies
the name of an existing \fIsites\fR list file, name for a \fIvect\fR
file, and name of an \fIoutput\fR file using the form
.LP
.RS
\fBs.medp \fBsites\*=\fIname \fBvect\*=\fIname \fBoutput\*=\fIname\fR 
.RE
.LP
Alternately, the user can simply type \fBs.medp\fR on the command line,
without program arguments.  In this case, the user will be prompted for
parameter values using the standard GRASS parser interface
described in the manual entry for \fIparser\fR.
.SH NOTES
When using the \fBreport\fR option, the tabular report may be more
than 128 columns wide. Therefore, it may be useful to add a \*(TX
or \*(Ps wrapper before printing (e.g., 
enscript -h -r -fCourier-Bold5 -p report.ps report) so that the
page is in landscape  orientation and a very small font is used.
.LP
The input vector file defining the grid should be large enough so
that the extreme east-west and north-south nodes (for non-rotated grids) can
be used to store row and column effects. Otherwise, when residuals
\fIand\fR effects are written to the output file, residuals may share
the same spatial location as effects when written to a vector output file.
.LP
Currently, nothing is done with the residuals, but I would like
implement some sort of analysis of these in the future (suggestions?).
Kriging using these residuals as new data set is being planned.
.SH SEE ALSO
.I v.mkgrid, 
.I v.transform,
.I parser,
and
.br
Cressie, Noel A. C. (1991).
.I Statistics for Spatial Data,
New York, NY: John Wiley & Sons.
pp. 186-187.
.SH BUGS
This should still be consider beta-release software. Although I have
repeated results shown in Cressie (1991), I would greatly
appreciate someone testing this further.
.LP
The \fB-s\fR flag is yet to be implemented. It may not ever.
.LP
This should probably be re-written to use points in a vector file
as input (thus creating \fIv.medp\fR).
.LP
Ideally, I would also like to output three data files for plotting
by a non-native graphing program (e.g., GNUPLOT), but I never got
around to doing this. The classic plot is to show three surfaces:
data = trend + residuals.  The trend can be obtained by 
extrapolation and/or interpolation.
.LP
Extrapolating and interpolating using the row, column, and all effects
to create a raster file would be a nice feature to include in the
future. Instead, I am considering retrieving the effects from the
output file and creating the surface with separate program.
.LP
Please send all bug fixes and comments to the author.
.SH AUTHOR
James Darrell McCauley, Purdue University 
.if n .br 
(mccauley@ecn.purdue.edu)
