.TH v.reclass
.SH NAME
\fIv.reclass\fR \- Creates a new map layer whose category values
are based upon the user's reclassification of categories in an
existing vector map layer.
.br
.I (SCS GRASS Vector Program)
.SH SYNOPSIS
\fBv.reclass\fR
.br
\fBv.reclass help\fR
.br
\fBv.reclass [-d] input\fR=\fIname \fBoutput\fR=\fIname\fR \fBtype\fR=\fIarea, line, \fRor\fI site\fR [\fBtitle\fR=\fIname\fR]
.SH DESCRIPTION
\fIv.reclass\fR creates an \fIoutput\fR map layer
based on an \fIinput\fR vector map layer.
The output map layer will be a reclassification of the input map layer
based on reclass rules input to \fIv.reclass\fR.
A \fItitle\fR for the output map layer may be (optionally)
specified by the user.

The reclass rules are read from standard input (i.e., from the keyboard,
redirected from a file, or piped through another program).

.SH OPTIONS
The user can run this program either non-interactively or interactively.
The program will be run non-interactively if the user specifies
program arguments and flag settings on the command line using the form:
.LP
.RS
\fBv.reclass [-d] input\fR=\fIname \fBoutput\fR=\fIname\fR \fBtype\fR=\fIarea, line,\fR or \fIsite\fR [\fBtitle\fR=\fIname\fR]
.RE
.LP
After the user types in the above information on the command line,
the program will (silently) prompt the user for reclass rules to
be applied to the input map layer categories.
The form of these rules is described in further detail in the sections
on non-interactive program use reclass rules and examples, below.

.LP
\fBParameters:\fR
.IP \fB-d\fR 18
Dissolve common boundaries between areas
.IP \fBinput\fR=\fIname\fR 18
Vector input map \fIname\fR.
.IP \fBoutput\fR=\fIname\fR 18
Vector output map \fIname\fR.
.IP \fBtype\fR=\fIname\fR 18
Select type of vector input file.
.br
Options:  area, line, site

.LP
Alternately, the user can simply type \fBv.reclass\fR on the command line,
without program arguments.  In this case, the user will be prompted for
all needed inputs.

Before using \fBv.reclass\fR one must know the following:
.IP 1 5
The new categories desired;  and, which old categories fit into 
which new categories.
.IP 2 5
The names of the new categories.
.SH INTERACTIVE PROGRAM USE:  EXAMPLE
Suppose we want to reclassify the vector map layer \fIroads\fR,
consisting of five categories, into the three new categories:
paved roads, unpaved roads, and railroad tracks.
The user is asked whether the reclass table is to be established
with each category value initially set to 0, or with each category
value initially set to its own value.
A screen like that shown below then appears, listing the categories
of the \fIroads\fR vector map layer to be reclassified and prompting
the user for the new category values to be assigned them.

.if t \fB
.ne 15
.TS
center;
c s s s
c s s s
l l l l.
ENTER NEW CATEGORY NUMBERS FOR THESE CATEGORIES

OLD CATEGORY NAME	OLD	NEW	 
	NUM	NUM
no data	0	0__
Hard Surface, 2 lanes	1	0__
Loose Surface, 1 lane	2	0__
Improved Dirt	3	0__
Unimproved Dirt Trail	4	0__
Railroad, single track	5	0__

.T&
c s s s.
AFTER COMPLETING ALL ANSWERS, HIT <ESC> TO CONTINUE
(OR <Ctrl-C> TO CANCEL)
.TE
\fR

.LP
In the following screen the new category values have been entered
beside the appropriate old category names.  
Cells assigned category values 2, 3, and 4 in the old vector
map layer are now assigned the new category value 2 in the reclassed map;
cell data formerly assigned to category value 5 in the old vector map
map are now assigned the new category value 3 in the reclassed map.

.if t \fB
.ne 15
.TS
center;
c s s s
c s s s
l l l l.
ENTER NEW CATEGORY NUMBERS FOR THESE CATEGORIES

OLD CATEGORY NAME	OLD	NEW	 
	NUM	NUM
no data	0	0__
Hard Surface, 2 lanes	1	1__
Loose Surface, 1 lane	2	2__
Improved Dirt	3	2__
Unimproved Dirt Trail	4	2__
Railroad, single track	5	3__

.T&
c s s s.
AFTER COMPLETING ALL ANSWERS, HIT <ESC> TO CONTINUE
(OR <Ctrl-C> TO CANCEL)
.TE
\fR

.LP
Hitting the escape key <ESC> will bring up the following screen, which
prompts the user to enter a new title and category label for the newly
.B reclassed
categories.

.if t \fB
.ne 15
.TS
center;
c s s
l s s
l s s
c l l.
ENTER NEW CATEGORY NAMES FOR THESE CATEGORIES

TITLE:  Roads Reclassified
CAT	NEW CATEGORY NAME	 
NUM
0	no data
1	Paved Roads
2	Unpaved Roads
3	Railroad, single track

.T&
c s.
AFTER COMPLETING ALL ANSWERS, HIT <ESC> TO CONTINUE
(OR <Ctrl-C> TO CANCEL)
.TE
\fR

Based upon the information supplied by the user in the above sample screens,
the new output map and supporting category files are created.
.SH NON-INTERACTIVE PROGRAM USE:  RECLASS RULES
In non-interactive program use, the names of an input map, output map,
type of input map, 
and output map title are given on the command line.
However, the reclass rules are still read from standard input
(i.e., from the keyboard,
redirected from a file, or piped through another program).

Once the user has specified an input vector map layer,
output map layer name, type of input map layer, and (optionally) output map layer title by typing

.RS
\fBv.reclass [-d ] \fRinput\fR=\fIname \fBoutput\fR=\fIname\fR \fBtype\fR=\fIarea, line,\fR or \fIsite\fR [\fBtitle\fR=\fIname\fR]
.RE

Each line of input must have the following format:

.RS
\fBinput_categories\fR=\fIoutput_category  \fR[\fIlabel\fR]
.RE

where the input lines specify the category values
in the input vector map layer to be reclassified to the
new \fIoutput_category\fR category value.
Specification of a \fIlabel\fR to be associated with
the new output map layer category is optional.  If specified, it is
recorded as the category label for the new category value.
The equal sign = is required.  The \fIinput_category(ies)\fR may consist
of single category values or a range of such values
in the format "\fIlow\fR thru \fIhigh\fR."
The word "thru" must be present.

A line containing only the word 
.B end
terminates the input.
.SH NON-INTERACTIVE PROGRAM USE:  EXAMPLES
The following examples may help clarify the reclass rules.

.IP 1 3
This example reclassifies categories 1, 3 and 5 in the input vector map layer
to category 1 with category label "poor quality" in the output map layer,
and reclassifies input vector map layer categories 2, 4, and 6
to category 2 with the label "good quality" in the output map layer.

.RS
.RS
.TS
l l l l.
1 3 5	\&\fR=	1	poor quality
2 4 6	\&\fR=	2	good quality
.TE
.RE
.RE

.IP 2
This example reclassifies input vector map layer categories 1 thru 10 to output 
map layer category 1, input map layer categories 11 thru 20 to output map layer
category 2, and input map layer categories 21 thru 30 to output map layer
category 3, all without labels.

.RS
.RS
.TS
l l l.
 1 thru 10	\&\fR=	1
11 thru 20	\&\fR=	2
21 thru 30	\&\fR=	3
.TE
.RE
.RE

.IP 3
Subsequent rules override previous rules.  Therefore, the below example
reclassifies input vector map layer categories 1 thru 19 and 51 thru 100
to category 1 in the output map layer,
input vector map layer categories 20 thru 24 and 26 thru 50 to
the output map layer category 2, and input vector map layer category 25
to the output category 3.

.RS
.RS
.TS
c l l l.
1 thru 100 	\&\fR=	1	poor quality
20 thru 50 	\&\fR=	2	medium quality
25	\&\fR=	3	good quality
.TE
.RE
.RE

.IP 4
The previous example could also have been entered as:

.ne 6
.RS
.RS
.TS
c l l l.
 1 thru 19  51 thru 100	\&\fR=	1	poor quality
20 thru 24  26 thru 50  	\&\fR=	2	medium quality
25	\&\fR=	3	good quality
.TE
.RE

or as:

.ne 6
.RS
.TS
c l l l.
 1 thru 19 	\&\fR=	1	poor quality
51 thru 100 	\&\fR=	1
20 thru 24 	\&\fR=	2
26 thru 50 	\&\fR=	2	medium quality
25 	\&\fR=	3	good quality
.TE
.RE
.RE

.LP
The final example was given to show how the labels are handled.  If a new
category value appears in more than one rule (as is the case with new 
category values 1 and 2),
the last label which was specified becomes the label for that category.
In this case the labels are assigned exactly as in the two previous examples.
.SH NOTES
The \fIv.reclass\fR program generates a new vector
map layer. The \fIv.support\fR program will have to be run on the newly
created vector map layer to build topology.
.br
The dissolve \fI[-d]\fR option will work on only those areas which are of the same
conversion category value.  If a map area is inside (island) a converted
area and is NOT converted to the same value, its boundaries are output
to the resultant map.
.SH BEWARE
Category values which are not explicitly reclassified to a new value
by the user will be reclassified to 0.
.SH "SEE ALSO"
.I v.support
.SH AUTHORS
Paul H. Fukuhara, USDA SCS, NCG-GSS
.br
R. L. Glenn, USDA SCS, NHQ-CGIS
