/*  %W%  %G%  */
/*

   Chris Rewerts, Agricultural Engineering, Purdue University
   April 1991

   This function copies the edited data from the temporary
   file to the new cell layer. Also installs colr, cats,
   and hist support files.

*/


#define GLOBAL
#include "edit.h"

make_new_cell_layer()
{
    struct History hist;
    CELL *cell;
    int cellfd;

    int tmpfd;
    int row;

/* open the new cell file to contain the edited version of
   the original cell layer. open our temporary file for read
   and copy its contents to the layer */

    G_set_window(&real_window);

    cellfd = G_open_cell_new(new_name);
    tmpfd = open(tempfile, 0);
    lseek(tmpfd, 0L, 0);

    cell = G_allocate_cell_buf();

    fprintf(stderr, "\n     +-------------------------------------------+\n");
    fprintf(stderr, "     |         Saving new cell layer             |\n");
    fprintf(stderr, "     +---------------------------------------");


    for (row = 0; row < real_nrows; row++) {
        if (read(tmpfd, cell, real_ncols * cellsize) != (real_ncols * cellsize))
            error(1,"error writing cell file during copy");
        G_put_map_row(cellfd, cell);
        G_percent(row, real_nrows, 5);
        }
    G_percent(100, 100, 5);
    fprintf(stderr, "\n");

    close(tmpfd);
    G_close_cell(cellfd);
    unlink(tempfile);

/* create and write cat, colr, and hist support files
   for the newly created layer */

    if (colr_ok)
    {
	G_write_colors (new_name, user_mapset, &colr);
	G_free_colors (&colr);
        colr_ok = 0;
    }
    if (cats_ok)
    {
	cats.num = G_number_of_cats (new_name, user_mapset);
	G_write_cats (new_name, &cats);
	G_free_cats (&cats);
        cats_ok = 0;
    }

/* construct some history information */
    sprintf(hist.mapid, "%s", G_date()) ;
    sprintf(hist.title, "%s", new_name) ;
    sprintf(hist.mapset, "%s", user_mapset) ;
    sprintf(hist.creator, "%s", G_whoami()) ;
    sprintf(hist.maptype, "cell") ;
    sprintf(hist.edhist[0],"Generated by Dedit program") ;
    sprintf(hist.edhist[1],"from %s in %s ", orig_name, orig_mapset) ;
    hist.edlinecnt=2 ;

/* write history */
    if (G_write_history(new_name, &hist) == -1)
        error(0,"could not write history");

}


