/*
 *	Let XX be D or F, and xx be double or float, 
 		for double or float representations.

	For a given xdr implementation on given hardware, we seek
	numbers XDR_XX_INFINITY such that

	 1) The xdr encode/decode process is idempotent for that number, eg
		xdr_encode_xx(xdr_encode_xx(XDR_XX_INFINITY)) 
			== XDR_XX_INFINITY
	 2) The xdr representation for that number is the ieee (positive)
	    infinity. 
	    Octal byte streams:
		double 177 360 000 000 000 000 000 000
		float  177 200 000 000
 */
#ifndef vms
#include 	<rpc/rpc.h>
#include	<stdio.h>
#else
#include	stdio.h
#include 	"types.h"
#include 	"xdr.h"
#endif
#define	FNAME	"inf.xdr"

#ifdef vax
#define	XDR_D_INFINITY	1.7014118346046923e+38
/* #define	XDR_F_INFINITY	2.93873588e-39 /* 0x00000080 */
#define	XDR_F_INFINITY	1.70141173e+38	/* 0xffff7fff */
#else
#define	XDR_D_INFINITY	1.797693134862315900e+308
#define	XDR_F_INFINITY	3.40282357e+38
#endif

main()
{
	double dd, *pdd ;
	float  ff, *pff ;
	unsigned long *ulpd, *ulpf ;
	
	FILE *fd ;
	XDR xdr ;

	dd = XDR_D_INFINITY ;
	pdd = &dd ;
	ulpd = (unsigned long *) &dd ;

	ff = XDR_F_INFINITY ;
	pff = &ff ;
	ulpf = (unsigned long *) &ff ;

	puts("\n\tXDR Floating Point Infinity Consistancy Check\n") ;
#define WRITE
#ifdef WRITE
	printf("0x%08x %08x double\t% -0.18e\n",
		ulpd[0], ulpd[1], dd ) ;
	printf("0x%08x          float\t% -0.8e \n",
		ulpf[0], ff ) ;

	fd = fopen(FNAME,"w") ;
	if(fd == NULL)
	{
		perror("fopen failed\n") ;
		exit(-1) ;
	}
	xdrstdio_create(&xdr,fd, XDR_ENCODE) ;
	
	xdr_double(&xdr,pdd) ; 
	xdr_float(&xdr,pff) ; 

	xdrstdio_create(&xdr,fd, XDR_FREE) ;
	fclose(fd) ;
#endif

#ifdef unix
	{
		static char command[256] = "od -b " ;
		strcat(command,FNAME) ;
		putchar('\n') ;
		puts(command) ;
		system(command) ;
		putchar('\n') ;
	}
#endif

	fd = fopen(FNAME,"r") ;
	if(fd == NULL)
	{
		perror("fopen failed\n") ;
		exit(-1) ;
	}
	xdrstdio_create(&xdr,fd, XDR_DECODE) ;

	xdr_double(&xdr,pdd) ;
	xdr_float(&xdr,pff) ; 
	printf("0x%08x %08x double\t% -0.18e\n",
		ulpd[0], ulpd[1], dd ) ;
	printf("0x%08x          float\t% -0.8e \n",
		ulpf[0], ff ) ;
	putchar('\n') ;
}
