/*********************************************************************
 *   Copyright 1989, University Corporation for Atmospheric Research
 *   See netcdf/README file for copying and redistribution conditions.
 *   $Header: /u2/ldm/src/netcdf/ncdump/RCS/ncdump.c,v 1.9 89/12/20 11:18:20 mschmidt Exp $
 *********************************************************************/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "netcdf.h"
#include "ncdump.h"
#include "void.h"		/* defines Void as void, where available */

extern Void *emalloc();		/* error-checking malloc */
extern vardata();		/* prints data values of a variable */

char *progname;
char *netcdf_name;
bool header_only;		/* if true, don't print any variable data */
bool coord_vals;		/*
				 * if true, print header and coordinate
				 * dimension values (values of variables
				 * that are also dimensions), but no other
				 * variable data
				 */

main(argc, argv)
int argc;
char *argv[];
{
    extern int optind;
    extern int opterr;
    extern char *optarg;
    int c;
    int i;
    Void do_ncdump();

    opterr = 1;
    progname = argv[0];

    netcdf_name = (char *) 0;	/* default is to construct from file name */
    header_only = false;	/* by default, print data as well as header */

    while ((c = getopt(argc, argv, "chn:")) != EOF)
	switch(c) {
	case 'h':		/* header but no data */
	    header_only = true;
	    break;
	case 'c':		/* header and data for coordinate dimensions */
	    coord_vals = true;
	    break;
	case 'n':		/*
				 * provide different name than derived from
				 * file name
				 */
	    netcdf_name = optarg;
	    break;
	case '?':
	    usage();
	    break;
	}

    argc -= optind;
    argv += optind;

    i = 0;

    do {		
	if (argc > 0)
	do_ncdump(argv[i], netcdf_name);
    } while (++i < argc);
    exit(0);
}

usage()
{
    (Void) fprintf(stderr,
	    "Usage: %s [ -h ] [ -c ] [ -n netcdf_name ] [ file ... ]\n", progname);
    exit(1);
}


/* 
 * convert pathname of netcdf file into name for cdl unit, by taking 
 * last component of path and stripping off any extension.
 */
static char *
name_path(path)
     char *path;
{
    char *cp, *new;

#ifdef unix
#define FILE_DELIMITER '/'
#endif
#ifdef vms
#define FILE_DELIMITER ']'
#endif    
#ifdef MSDOS
#define FILE_DELIMITER '\\'
#endif    
    cp = strrchr(path, FILE_DELIMITER);
    if (cp == 0)		/* no delimiter */
      cp = path;
    else			/* skip delimeter */
      cp++;
    new = (char *) emalloc(strlen(cp)+1);
    if (new == 0) {
	error("out of memory!");
	exit(1);
    }
    (Void) strcpy(new, cp);	/* copy last component of path */
    if (cp = strrchr(new, '.'))
      *cp = '\0';		/* strip off any extension */
    return new;
}


char *
type_name(type)
     nc_type type;
{
    switch (type) {
      case NC_BYTE:
	return "byte";
      case NC_CHAR:
	return "char";
      case NC_SHORT:
	return "short";
      case NC_LONG:
	return "long";
      case NC_FLOAT:
	return "float";
      case NC_DOUBLE:
	return "double";
      default:
	error("type_name: bad type %d", type);
	return "bogus";
    }
}


/* strips trailing zeros after decimal point from number string, in place */
char *
stz(ns)
     char *ns;			/* floating or double number string */
{
    register char *sp, *tp;

    if((sp = strchr(ns, '.')) == 0)
      return ns;
    /* ns has a decimal point, find next non-numeric (or end of string) */
    while (isdigit(*++sp))
      ;
    tp = sp - 1;		/* sp is first non-digit after decimal point */
    while (*tp == '0')
      tp--;
				/* tp is last non-zero after decimal point */
    if (sp == tp+1)
      return ns;		/* no trailing zeros after decimal point */
    while (*++tp = *sp)
      sp++;			/* copy over trailing zeros */
    return ns;
}


/* print list of attribute values.  Attribute values must be printed with
   explicit type tags, because their types are not declared. */
Void
pr_att_vals(type, len, vals)
     nc_type type;
     int len;
     Void *vals;
{
    int iel;
    union {
	char *cp;
	short *sp;
	long *lp;
	float *fp;
	double *dp;
    } gp;
    char c, *sp;
    char gps[30];		/* for ascii of a float or double precision */

    switch (type) {
      case NC_BYTE:
	gp.cp = (char *) vals;
	for (iel = 0; iel < len; iel++)
	  if (isprint(c = *gp.cp++ & 0377))
	    Printf ("'%c'%s", c, iel<len-1 ? ", " : "");
	  else
	    Printf ("'\\%o'%s", c, iel<len-1 ? ", " : "");
	break;
      case NC_CHAR:
	gp.cp = (char *) vals;
	Printf ("\"");
	/* adjust len so trailing nulls don't get printed */
	sp = gp.cp + len - 1;
	while (*sp-- == '\0' && len > 0)
	    len--;
	for (iel = 0; iel < len; iel++)
	  switch (c = *gp.cp++ & 0377) {
	    case '\b':
	      Printf ("\\b");
	      break;
	    case '\f':
	      Printf ("\\f");
	      break;
	    case '\n':
	      Printf ("\\n");
	      break;
	    case '\r':
	      Printf ("\\r");
	      break;
	    case '\t':
	      Printf ("\\t");
	      break;
	    case '\v':
	      Printf ("\\v");
	      break;
	    case '\\':
	      Printf ("\\\\");
	      break;
	    case '\'':
	      Printf ("\\'");
	      break;
	    case '\"':
	      Printf ("\\\"");
	      break;
	    default:
	      Printf ("%c",c);
	      break;
	  }
	Printf ("\"");
	break;
      case NC_SHORT:
	gp.sp = (short *) vals;
	for (iel = 0; iel < len; iel++)
	  Printf ("%ds%s",*gp.sp++,iel<len-1 ? ", " : "");
	break;
      case NC_LONG:
	gp.lp = (long *) vals;
	for (iel = 0; iel < len; iel++)
	  Printf ("%ld%s",*gp.lp++,iel<len-1 ? ", " : "");
	break;
      case NC_FLOAT:
	gp.fp = (float *) vals;
	for (iel = 0; iel < len; iel++) {
	    /* make sure we get a decimal point for float type attributes */
	    if (*gp.fp != 0.0)
	      sprintf(gps,"%#.8gf",*gp.fp++);
	    else		/* want "0.f" not "0f" from bug in sprintf() */
	      sprintf(gps,"%#.8ff",*gp.fp++);
	    stz(gps);		/* delete trailing zeros after decimal */
	    Printf ("%s%s", gps, iel<len-1 ? ", " : "");
	}
	break;
      case NC_DOUBLE:
	gp.dp = (double *) vals;
	for (iel = 0; iel < len; iel++) {
	    /* make sure we get a decimal point for float type attributes */
	    if (*gp.dp != 0)
	      sprintf(gps,"%#.17g",*gp.dp++);
	    else
	      sprintf(gps,"%#.17f",*gp.dp++);
	    stz(gps);		/* delete trailing zeros after decimal */
	    Printf ("%s%s", gps, iel<len-1 ? ", " : "");
	}
	break;
      default:
	error("pr_att_vals: bad type");
    }
}


Void
do_ncdump(path, name)
     char *path;
     char *name;
{
    int cdfid;
    int ndims;			/* number of dimensions */
    int nvars;			/* number of variables */
    int ngatts;			/* number of global attributes */
    int xdimid;			/* id of unlimited dimension */
    int dimid;			/* dimension id */
    int varid;			/* variable id */
    struct ncdim dims[MAX_NC_DIMS]; /* dimensions */
    int vdims[MAX_NC_DIMS];	/* dimension sizes for a single variable */
    struct ncvar var;		/* variable */
    struct ncatt att;		/* attribute */
    int id;			/* dimension number per variable */
    int ia;			/* attribute number */
    int is_coord;		/* true if variable is a coordinate variable */

    /* open netcdf file */
    if((cdfid = ncopen(path, NC_NOWRITE)) == -1) {
	error("ncopen failed on %s", path);
	return;
    }
    /* if name not specified, derive it from path */
    if (name == (char *)0) {
	name = name_path (path);
    }

    Printf ("netcdf %s {\n", name);
    /*
     * get number of dimensions, number of variables, number of global
     * atts, and dimension id of unlimited dimension, if any
     */
    if (ncinquire(cdfid, &ndims, &nvars, &ngatts, &xdimid) == -1) {
	error("ncinquire failed");
	(Void) ncclose(cdfid);
	return;
    }
    /* get dimension info */
    if (ndims > 0)
      Printf ("dimensions:\n");
    for (dimid = 0; dimid < ndims; dimid++) {
	if (ncdiminq(cdfid, dimid, dims[dimid].name, &dims[dimid].size) == -1) {
	    error("ncdiminq failed on dim id %d", dimid);
	    (Void) ncclose(cdfid); return;
	}
	if (dimid == xdimid)
	  Printf ("\t%s = %s ; // (%d currently)\n",dims[dimid].name,
		  "UNLIMITED", dims[dimid].size);
	else
	  Printf ("\t%s = %d ;\n",dims[dimid].name, dims[dimid].size);
    }

    Printf ("\nvariables:\n");
    /* get variable info, with variable attributes */
    for (varid = 0; varid < nvars; varid++) {
	if (ncvarinq(cdfid, varid, var.name, &var.type,
		      &var.ndims, var.dims, &var.natts) == -1) {
	    error("ncvarinq failed on var id %d", varid);
	    (Void) ncclose(cdfid); return;
	}
	Printf ("\t%s %s", type_name(var.type), var.name);
	if (var.ndims > 0)
	  Printf ("(");
	for (id = 0; id < var.ndims; id++) {
	    Printf ("%s%s",
		    dims[var.dims[id]].name,
		    id < var.ndims-1 ? ", " : ")");
	}
	Printf (" ;\n");
	/* get variable attributes */
	for (ia = 0; ia < var.natts; ia++) {
	    if (ncattname(cdfid, varid, ia, att.name) == -1) {
		error("ncattname failed on variable attribute");
		(Void) ncclose(cdfid);
		return;
	    }
	    
	    Printf ("\t\t%s:%s = ", var.name, att.name);

	    if (ncattinq(cdfid, varid, att.name, &att.type, &att.len) == -1) {
		error("ncattinq of variable attribute failed");
		(Void) ncclose(cdfid);
		return;
	    }
	    att.val = emalloc(att.len * nctypelen(att.type));
	    if (ncattget(cdfid, varid, att.name, att.val) == -1) {
		error("ncattget of variable attribute failed");
		(Void) ncclose(cdfid);
		return;
	    }
	    (Void) pr_att_vals(att.type, att.len, att.val);
	    Printf (" ;\n");
	    free ((char *) att.val);
	}
    }

    /* get global attributes */
    if (ngatts > 0)
      Printf ("\n// global attributes:\n");
    for (ia = 0; ia < ngatts; ia++) {
	if (ncattname(cdfid, NC_GLOBAL, ia, att.name) == -1) {
	    error("ncattname failed on global attribute");
	    (Void) ncclose(cdfid);
	    return;
	}
	
	Printf ("\t\t:%s = ", att.name);
	
	if (ncattinq(cdfid, NC_GLOBAL, att.name, &att.type, &att.len) == -1) {
	    error("ncattinq of global attribute failed");
	    (Void) ncclose(cdfid);
	    return;
	}
	att.val = emalloc(att.len * nctypelen(att.type));
	if (ncattget(cdfid, NC_GLOBAL, att.name, att.val) == -1) {
	    error("ncattget of global attribute failed");
	    (Void) ncclose(cdfid);
	    return;
	}
	(Void) pr_att_vals(att.type, att.len, att.val);
	Printf (" ;\n");
	free ((char *) att.val);
    }
    
    if (! header_only) {
	if (nvars > 0)
	  Printf ("\ndata:\n");
	/* output variable data */
	for (varid = 0; varid < nvars; varid++) {
	    if (ncvarinq(cdfid, varid, var.name, &var.type,
			 &var.ndims, var.dims, &var.natts) == -1) {
		error("ncvarinq failed on var id %d", varid);
		(Void) ncclose(cdfid); return;
	    }
	    if (coord_vals) {
		/* Find out if this is a coordinate variable */
		is_coord = 0;
		for (dimid = 0; dimid < ndims; dimid++) {
		    if (strcmp(dims[dimid].name, var.name) == 0 &&
			var.ndims == 1) {
			is_coord = 1;
			break;
		    }
		}
		if (! is_coord)	/* don't get data for non-coordinate vars */
		  continue;
	    }
	    /*
	     * Only get data for variable if it is not a record variable,
	     * or if it is a record variable and at least one record has
	     * been written.
	     */
	    if (var.dims[0] != xdimid || dims[xdimid].size != 0) {
		/* Collect variable's dim sizes */
		for (id = 0; id < var.ndims; id++)
		  vdims[id] = dims[var.dims[id]].size;
		if (vardata(&var,vdims,cdfid,varid) == -1) {
		    error("can't output data for variable %s", var.name);
		    (Void) ncclose(cdfid); return;
		}
	    }
	}
    }

    Printf ("}\n");
    if (ncclose(cdfid) == -1)
      Printf ("ncclose failed");
}
