/*********************************************************************
 *   Copyright 1989, University Corporation for Atmospheric Research
 *   See netcdf/README file for copying and redistribution conditions.
 *   $Header: /u2/ldm/src/netcdf/ncdump/RCS/vardata.c,v 1.6 89/11/25 21:51:38 russ Exp $
 *********************************************************************/

#include <stdio.h>
#include <ctype.h>
#include "netcdf.h"
#include "ncdump.h"

extern Void *emalloc();		/* error-checking malloc */
extern Void set_indent();	/* set postion in line before lput() calls */
extern Void lput();		/* splits lines to keep them short */

/*
 * Print a row of variable values.  Makes sure output lines aren't too long
 * by judiciously inserting newlines. 
 */
static Void
pr_vals(type, len, vals)
     nc_type type;		/* one of NC_BYTE, ..., NC_DOUBLE */
     int len;			/* number of values to print */
     Void *vals;		/* pointer to block of values */
{
    int iel;
    union {
	char *cp;
	short *sp;
	long *lp;
	float *fp;
	double *dp;
    } gp;
    char c, *sp;
    float fill_float;
    double fill_double;
    char sout[25];		/* temporary string for each encoded output */

    fill_float = FILL_FLOAT;	/* static initialization hits ultrix cc bug */
    fill_double = FILL_DOUBLE;
    switch (type) {
      case NC_BYTE:
	gp.cp = (char *) vals;
	for (iel = 0; iel < len; iel++) {
	    (void) sprintf (sout, "0%o%s",*gp.cp++ & 0377, iel<len-1 ? ", " : "");
	    lput (sout);
	}
	break;
      case NC_CHAR:
	gp.cp = (char *) vals;
	Printf ("\"");
	/* adjust len so trailing nulls don't get printed */
	sp = gp.cp + len - 1;
	while (*sp-- == '\0' && len > 0)
	    len--;
	for (iel = 0; iel < len; iel++)
	  switch (c = *gp.cp++ & 0377) {
	    case '\b':
	      Printf ("\\b");
	      break;
	    case '\f':
	      Printf ("\\f");
	      break;
	    case '\n':
	      Printf ("\\n");
	      break;
	    case '\r':
	      Printf ("\\r");
	      break;
	    case '\t':
	      Printf ("\\t");
	      break;
	    case '\v':
	      Printf ("\\v");
	      break;
	    case '\\':
	      Printf ("\\\\");
	      break;
	    case '\'':
	      Printf ("\\\'");
	      break;
	    case '\"':
	      Printf ("\\\"");
	      break;
	    default:
	      if (isprint(c))
		Printf ("%c",c);
	      else
		Printf ("\\%.3o",c);
	      break;
	  }
	Printf ("\"");
	break;
      case NC_SHORT:
	gp.sp = (short *) vals;
	for (iel = 0; iel < len; iel++) {
	    (void) sprintf (sout, "%d%s",*gp.sp++,iel<len-1 ? ", " : "");
	    lput (sout);
	}
	break;
      case NC_LONG:
	gp.lp = (long *) vals;
	for (iel = 0; iel < len; iel++) {
	    (void) sprintf (sout, "%ld%s",*gp.lp++,iel<len-1 ? ", " : "");
	    lput (sout);
	}
	break;
      case NC_FLOAT:
	gp.fp = (float *) vals;
	for (iel = 0; iel < len; iel++) {
	    if (*gp.fp >= fill_float)
	      (void) sprintf (sout, "FloatInf%s",iel<len-1 ? ", " : "");
	    else
	      (void) sprintf (sout, "%.8g%s",*gp.fp,iel<len-1 ? ", " : "");
	    lput (sout);
	    gp.fp++;
	}
	break;
      case NC_DOUBLE:
	gp.dp = (double *) vals;
	for (iel = 0; iel < len; iel++) {
	    if (*gp.dp >= fill_double)
	      (void) sprintf (sout, "DoubleInf%s",iel<len-1 ? ", " : "");
	    else
	      (void) sprintf (sout, "%.17g%s",*gp.dp,iel<len-1 ? ", " : "");
	    lput (sout);
	    gp.dp++;
	}
	break;
      default:
	error("pr_vals: bad type");
    }
}


int
vardata(vp, vdims, cdfid, varid)
    struct ncvar *vp;		/* variable */
    int vdims[];		/* variable dimension sizes */
    int cdfid;			/* netcdf id */
    int varid;			/* variable id */
{
    int cor[MAX_NC_DIMS];	/* corner coordinates */
    int edg[MAX_NC_DIMS];	/* edges of hypercube */
    Void *vals;			/* pointer to variable values */
    int id;
    int ir;
    int nels;
    int ncols;
    int nrows;
    int vrank;
    char *cp;

    vrank = vp->ndims;

    if (vrank <= 1) {
	Printf ("\n %s = ", vp->name);
	set_indent (strlen(vp->name) + 4);
    } else {
	Printf ("\n %s =\n  ", vp->name);
	set_indent (2);
    }


    nels = 1;
    for (id = 0; id < vrank; id++) {
	cor[id] = 0;
	edg[id] = vdims[id];
	nels *= vdims[id];	/* total number of values for variable */
    }
    if (vrank >= 1)
      ncols = vdims[vrank-1];	/* size of "row" along last dimension */
    else
      ncols = 1;
    nrows = nels/ncols;		/* number of "rows" */

    vals = emalloc(nels * nctypelen(vp->type));
    if(ncvarget (cdfid, varid, cor, edg, vals) == -1)
      error("ncvarget failed for variable %s", vp->name);
    cp = (char *)vals;
    for (ir = 0; ir < nrows; ir++) {
	(Void) pr_vals(vp->type, ncols, (Void *)cp);
	cp += ncols * nctypelen(vp->type);
	Printf ("%s", ir<nrows-1 ? ",\n  " : " ;\n");
	set_indent(2);
    }

    free ((char *) vals);
    return 0;
}
