/*********************************************************************
 *   Copyright 1989, University Corporation for Atmospheric Research
 *   See netcdf/README file for copying and redistribution conditions.
 *   $Header: /u2/ldm/src/netcdf/ncgen/RCS/cdf_getfill.c,v 1.1 89/05/04 09:30:58 russ Exp $
 *********************************************************************/

#include "netcdf.h"
#include "ncvoid.h"
#include "generic.h"

/*
 * Given netCDF type, return a fill_value appropriate for
 * that type.
 */
Void
cdf_getfill(type, gval)
     nc_type type;
     union generic *gval;
{
    extern Void derror();

    switch(type) {
      case NC_CHAR:
	gval->charv = FILL_CHAR;
	return;
      case NC_BYTE:
	gval->charv = FILL_BYTE;
	return;
      case NC_SHORT:
	gval->shortv = FILL_SHORT;
	return;
      case NC_LONG:
	gval->longv = FILL_LONG;
	return;
      case NC_FLOAT:
	gval->floatv = FILL_FLOAT;
	return;
      case NC_DOUBLE:
	gval->doublev = FILL_DOUBLE;
	return;
      default:
	derror("cdffillval: unrecognized type");
    }
}

Void
cdf_fill(type, num, datp, fill_val)
     nc_type type;			/* netcdf type code  */
     int num;			/* number of values to fill */
     Void *datp;		/* where to start filling */
     union generic fill_val;	/* value to use */
{
    char *char_valp;		/* pointers used to accumulate data values */
    short *short_valp;
    long *long_valp;
    float *float_valp;
    double *double_valp;

    switch (type) {
      case NC_CHAR:
      case NC_BYTE:
	char_valp = (char *) datp;
	break;
      case NC_SHORT:
	short_valp = (short *) datp;
	break;
      case NC_LONG:
	long_valp = (long *) datp;
	break;
      case NC_FLOAT:
	float_valp = (float *) datp;
	break;
      case NC_DOUBLE:
	double_valp = (double *) datp;
	break;
    }
    while (num--) {
	switch (type) {
	  case NC_CHAR:
	  case NC_BYTE:
	    *char_valp++ = fill_val.charv;
	    break;
	  case NC_SHORT:
	    *short_valp++ = fill_val.shortv;
	    break;
	  case NC_LONG:
	    *long_valp++ = fill_val.longv;
	    break;
	  case NC_FLOAT:
	    *float_valp++ = fill_val.floatv;
	    break;
	  case NC_DOUBLE:
	    *double_valp++ = fill_val.doublev;
	    break;
	}
    }
}
