/*********************************************************************
 *   Copyright 1989, University Corporation for Atmospheric Research
 *   See netcdf/README file for copying and redistribution conditions.
 *   $Header: /u2/ldm/src/netcdf/ncgen/RCS/generate.c,v 1.2 89/11/06 08:26:51 russ Exp $
 *********************************************************************/

#include <stdio.h>
#include <string.h>
#include "ncgen.h"

extern Void derror();
extern Void *emalloc();

extern char *netcdf_name; /* output netCDF filename, if on command line. */

char *cdfctypename();
char *cdfftypename();
char *cdftypename();
char *ftypename();
char *cdfconststr();
char *cdfstrstr();

extern int netcdf_flag;
extern int c_flag;
extern int fortran_flag;

/* create netCDF from in-memory structure */
static Void
gen_netcdf(netcdfname)
     char *netcdfname;
{
    char *filename;
    int idim, ivar, iatt;
    int istat;
    int dimids[MAX_NC_DIMS];
    int varids[MAX_NC_VARS];

    if (netcdf_name) {		/* name given on command line */
	filename = netcdf_name;
    } else {			/* construct name from CDL name */
	filename = (char *) emalloc(strlen(netcdfname) + 5);
	*filename = '\0';
	(Void) strcat(filename,netcdfname);
	(Void) strcat(filename,".cdf");
    }

    cdfid = nccreate (filename, NC_CLOBBER);

    /* define dimensions from info in dims array */
    for (idim = 0; idim < ndims; idim++) {
	dimids[idim] = ncdimdef(cdfid, dims[idim].name, dims[idim].size);
	if (dimids[idim] == -1)
	  derror("error defining dimension %s of size %ld",
		 dims[idim].name,dims[idim].size);
    }

    /* define variables from info in vars array */
    for (ivar = 0; ivar < nvars; ivar++) {
	varids[ivar] = ncvardef (cdfid,
				  vars[ivar].name,
				  vars[ivar].type,
				  vars[ivar].ndims,
				  vars[ivar].dims);
	if (varids[ivar] == -1)
	  derror("error defining variable %s", vars[ivar].name);
    }

    /* define attributes from info in atts array */
    for (iatt = 0; iatt < natts; iatt++) {
	istat = ncattput(cdfid,
			  (atts[iatt].var == -1) ? NC_GLOBAL : atts[iatt].var,
			  atts[iatt].name,
			  atts[iatt].type,
			  atts[iatt].len,
			  atts[iatt].val);
	if (istat == -1)
	  derror("error defining attribute %s for variable %s",
		 atts[iatt].name,vars[atts[ivar].var].name);
    }

    istat = ncendef (cdfid);
    if (istat == -1) {
	derror("error returned from ncendef, exiting ...");
	exit(1);
    }
}

FILE *cout;			/* for C output */
FILE *fout;			/* for fortran output */

#define fpr    (Void) fprintf

/* create C code for creating netCDF from in-memory structure */
Void
gen_c(netcdfname)
     char *netcdfname;
{
    int idim, ivar, iatt, jatt, itype;
    char *val_string;
    int scalar_atts, vector_atts;

    static char *ctypes[] = {"char","short","long","float","double"};
    int ntypes = (sizeof ctypes) / (sizeof ctypes[0]);

    cout = stdout;

    /* wrap in procedure "create_netcdfname" */
    fpr(cout, "#include \"netcdf.h\"\n\nvoid\ncreate_%s() {\n", netcdfname);

    /* create necessary declarations */
    fpr(cout, "\n   int  cdfid;\t\t\t/* netCDF id */\n");

    if (ndims > 0) {
	fpr(cout, "\n   /* dimension ids */\n");
	fpr(cout, "   int  ");
    }
    for (idim = 0; idim < ndims; idim++)
	fpr(cout, "%s_dim%s", dims[idim].name, idim == ndims-1 ? ";\n" : ", ");

    if (nvars > 0) {
	fpr(cout, "\n   /* variable ids */\n");
	fpr(cout, "   int  ");
    }
    for (ivar = 0; ivar < nvars; ivar++)
	fpr(cout, "%s_id%s",
	    vars[ivar].name,
	    ivar == nvars-1 ? ";\n" : ", ");

    if (nvars > 0) {
	fpr(cout, "\n   /* variable shapes */\n");
	fpr(cout, "   int  ");
    }
    for (ivar = 0; ivar < nvars; ivar++)
	fpr(cout, "%s_dims[%d]%s",
	    vars[ivar].name,
	    vars[ivar].ndims,
	    ivar == nvars-1 ? ";\n" : ", ");

    /* determine if any containers for scalar attributes needed */
    scalar_atts = 0;
    for (iatt = 0; iatt < natts; iatt++) {
	if (atts[iatt].len == 1) {
	    scalar_atts = 1;
	    break;
	}
    }
    if (scalar_atts) {
	fpr(cout, "\n   /* containers for scalar attributes */\n");
	for (itype = 0; itype < ntypes; itype++) {
	    for (iatt = 0; iatt < natts; iatt++) {
		char type_name[12]; /* big enough for longest c typename */
		(Void) strcpy(type_name,cdfctypename(atts[iatt].type));
		if (atts[iatt].len == 1 &&
		    strcmp(type_name,ctypes[itype]) == 0) {
		    fpr(cout, "   %s  %s_val;\n", type_name, type_name);
		    break;
		}
	    }
	}
    }
    
    /* determine if we need any attribute vectors */
    vector_atts = 0;
    for (iatt = 0; iatt < natts; iatt++) {
	if (atts[iatt].len > 1 && atts[iatt].type != NC_CHAR) {
	    vector_atts = 1;
	    break;
	}
    }
    if (vector_atts) {
	fpr(cout, "\n   /* attribute vectors */\n");
	for (iatt = 0; iatt < natts; iatt++) {
	    if (atts[iatt].len > 1 && atts[iatt].type != NC_CHAR) {
		fpr(cout, 
		    "   %s  %s_%s[%d];\n",
		    cdfctypename(atts[iatt].type),
		    atts[iatt].var == -1 ? "cdf" : vars[atts[iatt].var].name,
		    atts[iatt].name,
		    atts[iatt].len);
	    }
	}
    }
    
    /* create netCDF file, uses NC_CLOBBER mode */
    fpr(cout, "\n   cdfid = nccreate(\"%s.cdf\", NC_CLOBBER);\n", netcdfname);
    
    /* define dimensions from info in dims array */
    if (ndims > 0)
      fpr(cout, "\n   /* define dimensions */\n");
    for (idim = 0; idim < ndims; idim++)
      if (dims[idim].size == NC_UNLIMITED)
	fpr(cout, "   %s_dim = ncdimdef(cdfid, \"%s\", NC_UNLIMITED);\n",
	    dims[idim].name,dims[idim].name);
      else
	fpr(cout, "   %s_dim = ncdimdef(cdfid, \"%s\", %d);\n",
	    dims[idim].name,dims[idim].name,dims[idim].size);

    /* define variables from info in vars array */
    if (nvars > 0)
      fpr(cout, "\n   /* define variables */\n");
    for (ivar = 0; ivar < nvars; ivar++) {
	for (idim = 0; idim < vars[ivar].ndims; idim++) {
	    fpr(cout, "   %s_dims[%d] = %s_dim;\n",
		vars[ivar].name,
		idim,
		dims[vars[ivar].dims[idim]].name);
	}
	fpr(cout, "   %s_id = ncvardef (cdfid, \"%s\", %s, %d, %s_dims);\n\n",
	    vars[ivar].name,
	    vars[ivar].name,
	    cdftypename(vars[ivar].type),
	    vars[ivar].ndims,
	    vars[ivar].name);
    }

    /* define attributes from info in atts array */
    if (natts > 0)
      fpr(cout, "\n   /* assign attributes */\n");
    for (iatt = 0; iatt < natts; iatt++) {
	if (atts[iatt].type == NC_CHAR && atts[iatt].len > 1) { /* string */
	    val_string = cdfstrstr((char *) atts[iatt].val);
	    fpr(cout,
		"   ncattput (cdfid, %s%s, \"%s\", NC_CHAR, %d, (void *)%s);\n",
		atts[iatt].var == -1 ? "NC_GLOBAL" : vars[atts[iatt].var].name,
		atts[iatt].var == -1 ? "" : "_id",
		atts[iatt].name,
		atts[iatt].len,
		val_string);
	    free (val_string);
	}
	else if (atts[iatt].len <= 1) {	/* scalar attribute */
	    val_string = cdfconststr(atts[iatt].type, atts[iatt].val, 0);
	    fpr(cout, "   %s_val = %s;\n",
		 cdfctypename(atts[iatt].type),
		 val_string);
	    fpr(cout,
		"   ncattput (cdfid, %s%s, \"%s\", %s, %d,(void *) &%s_val);\n",
		atts[iatt].var == -1 ? "NC_GLOBAL" : vars[atts[iatt].var].name,
		atts[iatt].var == -1 ? "" : "_id",
		atts[iatt].name,
		cdftypename(atts[iatt].type),
		atts[iatt].len,
		cdfctypename(atts[iatt].type));
	    free (val_string);
	}
	else {			/* vector attribute */
	    for (jatt = 0; jatt < atts[iatt].len ; jatt++) {
		val_string = cdfconststr(atts[iatt].type,atts[iatt].val,jatt);
		fpr(cout, "   %s_%s[%d] = %s;\n",
		    atts[iatt].var == -1 ? "cdf" : vars[atts[iatt].var].name,
		    atts[iatt].name,
		    jatt, 
		    val_string);
		free (val_string);
	    }
	    
	    fpr(cout,
		"   ncattput (cdfid, %s%s, \"%s\", %s, %d, (void *) %s_%s);\n",
		atts[iatt].var == -1 ? "NC_GLOBAL" : vars[atts[iatt].var].name,
		atts[iatt].var == -1 ? "" : "_id",
		atts[iatt].name,
		cdftypename(atts[iatt].type),
		atts[iatt].len,
		atts[iatt].var == -1 ? "cdf" : vars[atts[iatt].var].name,
		atts[iatt].name);
	}
    }
    fpr(cout, "\n   ncendef (cdfid);\n");
}


Void
gen_fortran(netcdfname)
     char *netcdfname;
{
    int idim, ivar, iatt, jatt;
    int vector_atts, have_data;
    char *val_string;

    fout = stdout;

    /* wrap in subroutine */
    fpr(fout, "      subroutine c%s()\n", netcdfname);
    fpr(fout, "\n      include \"netcdf.inc\"\n");

    /* create necessary declarations */
    fpr(fout, "\n      integer  iret\n");
    fpr(fout, "\n* netCDF id\n");
    fpr(fout, "      integer  cdfid\n");

    if (ndims > 0) {
	fpr(fout, "\n* dimension ids\n");
	fpr(fout, "      integer  ");
    }
    for (idim = 0; idim < ndims; idim++)
	fpr(fout, "%sdim%s", dims[idim].name, idim == ndims-1 ? "\n" : ", ");

    if (nvars > 0) {
	fpr(fout, "\n* variable ids\n");
	fpr(fout, "      integer  ");
    }
    for (ivar = 0; ivar < nvars; ivar++)
	fpr(fout, "%sid%s",
	    vars[ivar].name,
	    ivar == nvars-1 ? "\n" : ", ");

    if (nvars > 0) {
	fpr(fout, "\n* variable shapes\n");
	fpr(fout, "      integer  ");
    }
    for (ivar = 0; ivar < nvars; ivar++)
	fpr(fout, "%sdims(%d)%s",
	    vars[ivar].name,
	    vars[ivar].ndims,
	    ivar == nvars-1 ? "\n" : ", ");

    /* determine if we need declarations for variables to be initialized */
    have_data = 0;
    for (ivar = 0; ivar < nvars; ivar++)
      if (vars[ivar].has_data) {
	  have_data = 1;
	  break;
      }
    if (have_data) {
	fpr(fout,"\n* variables to be initialized, upper left corners and edge lengths\n");
	for (ivar = 0; ivar < nvars; ivar++)
	  if (vars[ivar].has_data) {
	      fpr(fout, "      %s %s()\n",
		  cdfftypename(vars[ivar].type),
		  vars[ivar].name);
	      fpr(fout, "      integer %s1(%d), %s2(%d)\n",
		  vars[ivar].name,
		  vars[ivar].ndims,
		  vars[ivar].name,
		  vars[ivar].ndims
		  );
	  }
    }

    /* determine if we need any attribute vectors */
    vector_atts = 0;
    for (iatt = 0; iatt < natts; iatt++) {
	if (atts[iatt].len > 1 && atts[iatt].type != NC_CHAR) {
	    vector_atts = 1;
	    break;
	}
    }
    if (vector_atts) {
	fpr(fout, "\n* attribute vectors\n");
	for (iatt = 0; iatt < natts; iatt++) {
	    if (atts[iatt].len > 1 && atts[iatt].type != NC_CHAR) {
		fpr(fout, 
		    "      %s  %s%s(%d)\n",
		    cdfftypename(atts[iatt].type),
		    atts[iatt].var == -1 ? "cdf" : vars[atts[iatt].var].name,
		    atts[iatt].name,
		    atts[iatt].len);
	    }
	}
    }
	
    /* create netCDF file, uses NC_CLOBBER mode */
    fpr(fout, "\n* enter define mode\n");
    fpr(fout, "      cdfid = nccre (\"%s.cdf\", NCCLOB, iret)\n",
	netcdfname);
    
    /* define dimensions from info in dims array */
    if (ndims > 0)
      fpr(fout, "\n* define dimensions\n");
    for (idim = 0; idim < ndims; idim++)
      if (dims[idim].size == NC_UNLIMITED)
	fpr(fout, "      %sdim = ncddef(cdfid, \"%s\", NCUNLIM, iret)\n",
	    dims[idim].name,dims[idim].name);
      else
	fpr(fout, "      %sdim = ncddef(cdfid, \"%s\", %d, iret)\n",
	    dims[idim].name,dims[idim].name,dims[idim].size);

    /* define variables from info in vars array */
    if (nvars > 0)
      fpr(fout, "\n* define variables\n");
    for (ivar = 0; ivar < nvars; ivar++) {
	for (idim = 0; idim < vars[ivar].ndims; idim++) {
	    fpr(fout, "      %sdims(%d) = %sdim\n",
		vars[ivar].name,
		vars[ivar].ndims - idim, /* reverse FORTRAN dimensions */
		dims[vars[ivar].dims[idim]].name);
	}
	fpr(fout, 
	    "      %sid = ncvdef (cdfid, \"%s\", %s, %d, %sdims, iret)\n\n",
	    vars[ivar].name,
	    vars[ivar].name,
	    ftypename(vars[ivar].type),
	    vars[ivar].ndims,
	    vars[ivar].name);
    }

    /* define attributes from info in atts array */
    if (natts > 0)
      fpr(fout, "\n* assign attributes\n");
    for (iatt = 0; iatt < natts; iatt++) { /* no attribute vector needed */
	if (atts[iatt].type == NC_CHAR && atts[iatt].len > 1) { /* string */
	    val_string = cdfstrstr((char *) atts[iatt].val);
	    fpr(fout, 
		"      call ncaptc (cdfid, %s%s, \"%s\", NCCHAR, %d, %s, iret)\n",
		 atts[iatt].var == -1 ? "NCGLOBAL" : vars[atts[iatt].var].name,
		 atts[iatt].var == -1 ? "" : "id",
		 atts[iatt].name,
		 atts[iatt].len,
		 val_string);
	    free (val_string);
	}
	else if (atts[iatt].len <= 1) {	/* scalar attribute */
	    val_string = cdfconststr(atts[iatt].type, atts[iatt].val, 0);
	    fpr(fout, 
		 "      call ncapt (cdfid, %s%s, \"%s\", %s, %d, %s, iret)\n",
		 atts[iatt].var == -1 ? "NCGLOBAL" : vars[atts[iatt].var].name,
		 atts[iatt].var == -1 ? "" : "id",
		 atts[iatt].name,
		 ftypename(atts[iatt].type),
		 atts[iatt].len,
		 val_string);
	    free (val_string);
	}
	else {			/* vector attribute */
	    for (jatt = 0; jatt < atts[iatt].len ; jatt++) {
		val_string = cdfconststr(atts[iatt].type,atts[iatt].val,jatt);
		fpr(fout, "      %s_%s(%d) = %s\n",
		    atts[iatt].var == -1 ? "cdf" : vars[atts[iatt].var].name,
		    atts[iatt].name,
		    jatt+1, 
		    val_string);
		free (val_string);
	    }
	    
	    fpr(fout, 
		 "      call ncapt (cdfid, %s%s, \"%s\", %s, %d, %s%s, iret)\n",
		 atts[iatt].var == -1 ? "NCGLOBAL" : vars[atts[iatt].var].name,
		 atts[iatt].var == -1 ? "" : "id",
		 atts[iatt].name,
		 ftypename(atts[iatt].type),
		 atts[iatt].len,
		 atts[iatt].var == -1 ? "cdf" : vars[atts[iatt].var].name,
		 atts[iatt].name);
	}
    }
    fpr(fout, "\n      call ncendf (cdfid, iret)\n");
}


/* return C name for netCDF type, given type code */
char *
cdftypename(type)
     nc_type type;			/* netCDF type code */
{
    switch (type) {
      case NC_BYTE:
	return "NC_BYTE";
      case NC_CHAR:
	return "NC_CHAR";
      case NC_SHORT:
	return "NC_SHORT";
      case NC_LONG:
	return "NC_LONG";
      case NC_FLOAT:
	return "NC_FLOAT";
      case NC_DOUBLE:
	return "NC_DOUBLE";
      default:
	derror("cdftypename: bad type code");
	return 0;
    }
}


/* return FORTRAN name for netCDF type, given type code */
char *
ftypename(type)
     nc_type type;			/* netCDF type code */
{
    switch (type) {
      case NC_BYTE:
	return "NCBYTE";
      case NC_CHAR:
	return "NCCHAR";
      case NC_SHORT:
	return "NCSHORT";
      case NC_LONG:
	return "NCLONG";
      case NC_FLOAT:
	return "NCFLOAT";
      case NC_DOUBLE:
	return "NCDOUBLE";
      default:
	derror("ftypename: bad type code");
	return 0;
    }
}


/* return C type name for netCDF type, given type code */

char *
cdfctypename(type)
     nc_type type;			/* netCDF type code */
{
    switch (type) {
      case NC_BYTE:
	return "char";
      case NC_CHAR:
	return "char";
      case NC_SHORT:
	return "short";
      case NC_LONG:
	return "long";
      case NC_FLOAT:
	return "float";
      case NC_DOUBLE:
	return "double";
      default:
	derror("cdfctypename: bad type code");
	return 0;
    }
}


/* return Fortran type name for netCDF type, given type code */

char *
cdfftypename(type)
     nc_type type;			/* netCDF type code */
{
    switch (type) {
      case NC_BYTE:
	return "BYTE";		/* or equally non-standard INTEGER*1 ? */
      case NC_CHAR:
	return "CHARACTER";
      case NC_SHORT:
	return "INTEGER*2";
      case NC_LONG:
	return "INTEGER";	/* should be non-standard INTEGER*4 ? */
      case NC_FLOAT:
	return "REAL";
      case NC_DOUBLE:
	return "DOUBLE PRECISION";
      default:
	derror("cdfctypename: bad type code");
	return 0;

    }
}


/*
 * Given a netcdf type, a pointer to a vector of values of that type,
 * and the index of the vector element desired, returns a pointer to a
 * malloced string representing the value.
 */

char *
cdfconststr(type,valp, num)
     nc_type type;			/* netCDF type code */
     Void *valp;		/* pointer to vector of values */
     int num;			/* element of vector desired */
{
    static char *cp;
    char *charp;
    char *bytep;
    short *shortp;
    long *longp;
    float *floatp;
    double *doublep;
#ifdef lint
    extern char *sprintf();
#endif

    switch (type) {
      case NC_CHAR:
	cp = (char *) emalloc (7);
	*cp++ = '\'';
	*cp++ = '\\';
	charp = (char *)valp;
	(Void) sprintf (cp, "%.3o", * (charp + num) & 0xff);
	cp += 3;
	*cp++ = '\'';
	*cp = '\0';
	return cp-6;

      case NC_BYTE:
	cp = (char *) emalloc (7);
	bytep = (char *)valp;
	(Void) sprintf(cp,"'\\%o'", * (bytep + num) & 0xff);
	return cp;

      case NC_SHORT:
	cp = (char *) emalloc (10);
	shortp = (short *)valp;
	(Void) sprintf(cp,"%d",* (shortp + num));
	return cp;

      case NC_LONG:
	cp = (char *) emalloc (20);
	longp = (long *)valp;
	(Void) sprintf(cp,"%ld",* (longp + num));
	return cp;

      case NC_FLOAT:
	cp = (char *) emalloc (20);
	floatp = (float *)valp;
	(Void) sprintf(cp,"%.8g",* (floatp + num));
	return cp;

      case NC_DOUBLE:
	cp = (char *) emalloc (20);
	doublep = (double *)valp;
	(Void) sprintf(cp,"%.16g",* (doublep + num));
	return cp;

      default:
	derror("cdfconststr: bad type code");
	return 0;
    }
}


/*
 * Given a pointer to a string, returns a pointer to a malloced string
 * representing  the string as a C constant.
 */

char *
cdfstrstr(valp)
     char *valp;		/* pointer to vector of values */
{
    static char *sp;
    char *cp;

    /* *** fix for escaped characters, e.g. tabs *** */

    sp = cp = (char *) emalloc (strlen(valp)+3);
    *cp++ = '"';
    while (*cp++ = *valp++);
    *--cp = '"';
    *++cp = '\0';
    return sp;
}


/* invoke netcdf calls (or generate C or Fortran code) to create netcdf
 * from in-memory structure. */
Void
define_netcdf(netcdfname)
     char *netcdfname;
{
    if (netcdf_flag)
      gen_netcdf(netcdfname);	/* create netcdf */
    if (c_flag)			/* create C code to create netcdf */
      gen_c(netcdfname);
    if (fortran_flag)		/* create Fortran code to create netcdf */
      gen_fortran(netcdfname);
}
