/*********************************************************************
 *   Copyright 1989, University Corporation for Atmospheric Research
 *   See netcdf/README file for copying and redistribution conditions.
 *   $Header: /u2/ldm/src/netcdf/ncgen/RCS/load_netcdf.c,v 1.2 89/05/04 09:38:43 russ Exp $
 *********************************************************************/

#include <stdio.h>
#include "ncgen.h"

extern Void derror();

extern int netcdf_flag;
extern int c_flag;
extern int fortran_flag;

/* invoke netcdf calls (or generate C or Fortran code) to load netcdf variable
 * from in-memory data.  Assumes following global variables set from yacc
 * parser:
 * int varnum        - number of variable to be loaded.
 * struct vars[varnum] - structure containing info on variable, specifically
 *                     name, type, ndims, dims, fill_value, has_data
 * int rec_dim       - id of record dimension, or -1 if none
 * struct dims[]     - structure containing name and size of dimensions.
 * int netcdf_record_number - number of current record for this variable.
 */
int
put_variable(rec_start)
     Void *rec_start;		/* points to data to be loaded  */
{
    Void load_netcdf(), gen_load_c(), gen_load_fortran();

    if (netcdf_flag)
      load_netcdf(rec_start);	/* put variable values (one record's worth) */
    if (c_flag)		/* create C code to put values */
      gen_load_c(rec_start);
    if (fortran_flag)		/* create Fortran code to put values */
      gen_load_fortran(rec_start);
}


Void
load_netcdf(rec_start)	/* write out record from in-memory structure */
     Void *rec_start;
{
    int idim;
    int istat;
    int coords[MAX_VAR_DIMS];
    int edges[MAX_VAR_DIMS];
    char *charvalp;
    short *shortvalp;
    long *longvalp;
    float *floatvalp;
    double *doublevalp;

    /* load values into variable */

    switch (vars[varnum].type) {
      case NC_CHAR:
      case NC_BYTE:
	charvalp = (char *) rec_start;
	break;
      case NC_SHORT:
	shortvalp = (short *) rec_start;
	break;
      case NC_LONG:
	longvalp = (long *) rec_start;
	break;
      case NC_FLOAT:
	floatvalp = (float *) rec_start;
	break;
      case NC_DOUBLE:
	doublevalp = (double *) rec_start;
	break;
    }
    /* initialize coords to upper left corner (0,0,0,...) */
    if (vars[varnum].dims[0] == rec_dim) {
	coords[0] = netcdf_record_number;
	edges[0] = 1;
    }
    else {
	coords[0] = 0;
	edges[0] = dims[vars[varnum].dims[0]].size;
    }

    for (idim = 1; idim < vars[varnum].ndims; idim++) {
	coords[idim] = 0;
	edges[idim] = dims[vars[varnum].dims[idim]].size;
    }
    
    switch (vars[varnum].type) {
      case NC_CHAR:
      case NC_BYTE:
	istat = ncvarput (cdfid,varnum,coords,edges,(Void *)charvalp);
	break;
      case NC_SHORT:
	istat = ncvarput (cdfid,varnum,coords,edges,(Void *)shortvalp);
	break;
      case NC_LONG:
	istat = ncvarput (cdfid,varnum,coords,edges,(Void *)longvalp);
	break;
      case NC_FLOAT:
	istat = ncvarput (cdfid,varnum,coords,edges,(Void *)floatvalp);
	break;
      case NC_DOUBLE:
	istat = ncvarput (cdfid,varnum,coords,edges,(Void *)doublevalp);
	break;
    }
    if (istat == -1)
      derror("error putting value for variable %s",vars[varnum].name);
}


extern FILE *cout;			/* for generated C code */
extern FILE *fout;			/* for fortran output */

#define fpr    (Void) fprintf


/* generate C to put netCDF record from in-memory data */
Void
gen_load_c(rec_start)
     Void *rec_start;
{
    int idim, ival;
    char *val_string;
    char *charvalp;
    short *shortvalp;
    long *longvalp;
    float *floatvalp;
    double *doublevalp;
    extern char *cdfstrstr(),  *cdfconststr(), *cdfctypename();

    fpr(cout, "\n   {\n");	/* put data declarations in their own block */

    /* initialize coords to upper left corner (rec_num,0,0,...) */
    fpr(cout, "   static int %s_start[] = {", vars[varnum].name);
    if (vars[varnum].dims[0] == rec_dim)
      fpr(cout, "%d%s",
	  netcdf_record_number,
	  0 < vars[varnum].ndims-1 ? ", " : "};\n");
    else
      fpr(cout, "%d%s",
	  0,
	  0 < vars[varnum].ndims-1 ? ", " : "};\n");
    for (idim = 1; idim < vars[varnum].ndims; idim++)
      fpr(cout, "%d%s",
	  0,
	  idim < vars[varnum].ndims-1 ? ", " : "};\n");

    /* initialize edge lengths from upper left corner */
    fpr(cout, "   static int %s_edges[] = {", vars[varnum].name);
    if (vars[varnum].dims[0] == rec_dim)
      fpr(cout, "%d%s",
	  1,
	  0 < vars[varnum].ndims-1 ? ", " : "};\n");
    else
      fpr(cout, "%d%s",
	  dims[vars[varnum].dims[0]].size,
	  0 < vars[varnum].ndims-1 ? ", " : "};\n");
    for (idim = 1; idim < vars[varnum].ndims; idim++)
      fpr(cout, "%d%s",
	  dims[vars[varnum].dims[idim]].size,
	  idim < vars[varnum].ndims-1 ? ", " : "};\n");

    /* load variable with data values using static initialization */
    fpr(cout, "   static %s %s[] = {",
	cdfctypename(vars[varnum].type),
	vars[varnum].name);

    switch (vars[varnum].type) {
      case NC_CHAR:
	charvalp = (char *) rec_start;
	val_string = cdfstrstr(charvalp);
	fpr(cout, "%s",val_string);
	break;
      case NC_BYTE:
	charvalp = (char *) rec_start;
	for (ival = 0; ival < var_len-1; ival++) {
	    val_string = cdfconststr(NC_BYTE,(Void *)charvalp++,0);
	    fpr(cout, "%s, ", val_string);
	    free(val_string);
	}
	for (;ival < var_len; ival++) {
	    val_string = cdfconststr(NC_BYTE,(Void *)charvalp++,0);
	    fpr(cout, "%s", val_string);
	    free(val_string);
	}
	break;
      case NC_SHORT:
	shortvalp = (short *) rec_start;
	for (ival = 0; ival < var_len-1; ival++)
	  fpr(cout, "%d, ", *shortvalp++);
	for (;ival < var_len; ival++)
	  fpr(cout, "%d", *shortvalp);
	break;
      case NC_LONG:
	longvalp = (long *) rec_start;
	for (ival = 0; ival < var_len-1; ival++)
	  fpr(cout, "%ld, ", *longvalp++);
	for (;ival < var_len; ival++)
	  fpr(cout, "%ld", *longvalp);
	break;
      case NC_FLOAT:
	floatvalp = (float *) rec_start;
	for (ival = 0; ival < var_len-1; ival++)
	  fpr(cout, "%.8g, ", *floatvalp++);
	for (;ival < var_len; ival++)
	  fpr(cout, "%.8g", *floatvalp);
	break;
      case NC_DOUBLE:
	doublevalp = (double *) rec_start;
	for (ival = 0; ival < var_len-1; ival++)
	  fpr(cout, "%.16g, ", *doublevalp++);
	for (;ival < var_len; ival++)
	  fpr(cout, "%.16g", *doublevalp);
	break;
    }
    fpr(cout, "};\n");
    fpr(cout,
	"   ncvarput(cdfid, %s_id, %s_start, %s_edges, (void *)%s);\n",
	vars[varnum].name,
	vars[varnum].name,
	vars[varnum].name,
	vars[varnum].name);

    fpr(cout, "   }\n");
}


Void
gen_load_fortran(rec_start)  /* make Fortran to put record */
     Void *rec_start;
{
    int idim, ival;
    char *val_string;
    char *charvalp;
    short *shortvalp;
    long *longvalp;
    float *floatvalp;
    double *doublevalp;
    extern char *cdfstrstr(),  *cdfconststr(), *cdfctypename();

    /* initialize coords to upper left corner (1,1,...,rec_num) */
    fpr(fout, "\n* %s upper left corner and edge lengths\n",
	vars[varnum].name);
    fpr(fout, "      data %s1 /",vars[varnum].name);
    /* reverse dimensions for FORTRAN */
    for (idim = vars[varnum].ndims; idim > 0; idim--)
      fpr(fout, "%d,", 1);
    if (vars[varnum].dims[0] == rec_dim)
      fpr(fout, "%d/\n", netcdf_record_number+1);
    else
      fpr(fout, "%d/\n", 1);

    /* initialize edge lengths from upper left corner */
    fpr(fout, "      data %s2 /",vars[varnum].name);
    /* reverse dimensions for FORTRAN */
    for (idim = vars[varnum].ndims; idim > 0; idim--)
      fpr(fout, "%d,",
	  dims[vars[varnum].dims[idim]].size);
    if (vars[varnum].dims[0] == rec_dim)
      fpr(fout, "%d/\n", 1);
    else
      fpr(fout, "%d/\n", dims[vars[varnum].dims[0]].size);

    /* load variable with data values  */
    fpr(fout, "      data %s /",vars[varnum].name);

    switch (vars[varnum].type) {
      case NC_CHAR:
	charvalp = (char *) rec_start;
	val_string = cdfstrstr(charvalp);
	fpr(fout, "%s",val_string);
	break;
      case NC_BYTE:
	charvalp = (char *) rec_start;
	for (ival = 0; ival < var_len-1; ival++) {
	    val_string = cdfconststr(NC_BYTE,(Void *)charvalp++,0);
	    fpr(fout, "%s, ", val_string);
	    free(val_string);
	}
	for (;ival < var_len; ival++) {
	    val_string = cdfconststr(NC_BYTE,(Void *)charvalp++,0);
	    fpr(fout, "%s", val_string);
	    free(val_string);
	}
	break;
      case NC_SHORT:
	shortvalp = (short *) rec_start;
	for (ival = 0; ival < var_len-1; ival++)
	  fpr(fout, "%d, ", *shortvalp++);
	for (;ival < var_len; ival++)
	  fpr(fout, "%d", *shortvalp);
	break;
      case NC_LONG:
	longvalp = (long *) rec_start;
	for (ival = 0; ival < var_len-1; ival++)
	  fpr(fout, "%ld, ", *longvalp++);
	for (;ival < var_len; ival++)
	  fpr(fout, "%ld", *longvalp);
	break;
      case NC_FLOAT:
	floatvalp = (float *) rec_start;
	for (ival = 0; ival < var_len-1; ival++)
	  fpr(fout, "%.8g, ", *floatvalp++);
	for (;ival < var_len; ival++)
	  fpr(fout, "%.8g", *floatvalp);
	break;
      case NC_DOUBLE:
	doublevalp = (double *) rec_start;
	for (ival = 0; ival < var_len-1; ival++)
	  fpr(fout, "%.16g, ", *doublevalp++);
	for (;ival < var_len; ival++)
	  fpr(fout, "%.16g", *doublevalp);
	break;
    }
    fpr(fout, "/\n");
    fpr(fout,
	"      call ncvpt (cdfid, %sid, %s1, %s2, %s, iret)\n",
	vars[varnum].name,
	vars[varnum].name,
	vars[varnum].name,
	vars[varnum].name);

}
