%{
/*********************************************************************
 *   Copyright 1989, University Corporation for Atmospheric Research
 *   See netcdf/README file for copying and redistribution conditions.
 *   $Header: /u2/ldm/src/netcdf/ncgen/RCS/ncgen.l,v 1.4 89/11/24 11:40:32 russ Exp $
 *********************************************************************/

#define	STREQ(a, b)	(*(a) == *(b) && strcmp((a), (b)) == 0)

/* lex specification for tokens for ncgen */

	extern double atof();
	extern long atol();
	extern long strtol();
	void expand_escapes();

#ifdef VMS
#include "ncgen_tab.h"
#else
#include "ncgen.tab.h"
#endif
%}

escquote	\\\"
nonquotes	([^"]|{escquote})*
%%
\/\/.*		/* comment */ ;

\"{nonquotes}\"		{
			 if(yyleng > MAXTRST) {
				yyerror("string too long, truncated\n");
			        yytext[MAXTRST-1] = '\0';
			 }
			 expand_escapes(termstring);
		 	 return (TERMSTRING);
		        }

float|FLOAT|real|REAL	{return (FLOAT_K);}
char|CHAR		{return (CHAR_K);}
byte|BYTE		{return (BYTE_K);}
short|SHORT		{return (SHORT_K);}
long|LONG|int|INT|integer|INTEGER	{return (LONG_K);}
double|DOUBLE		{return (DOUBLE_K);}
unlimited|UNLIMITED	{long_val = -1;
			 return (NC_UNLIMITED_K);}

dimensions:|DIMENSIONS:	{return (DIMENSIONS);}
variables:|VARIABLES:	{return (VARIABLES);}
data:|DATA:		{return (DATA);}
netcdf|NETCDF|netCDF	{return (NETCDF);}

DoubleInf	{    /* double missing values */
		double_val = FILL_DOUBLE;  /* IEEE double infinity */
		return (DOUBLE_CONST);
		}

FloatInf|Infinity|Inf	{    /* float missing values */
		float_val = FILL_FLOAT;  /* IEEE float infinity */
		return (FLOAT_CONST);
		}

[A-Za-z_][A-Za-z_0-9]*	{
		char *s;
		for(s=yytext;*s;s++)
			if(*s == '.')
				*s = '_';
		if (strlen(yytext) >= MAX_NC_NAME) {
		        derror("%s: name too long",yytext);
			yytext[MAX_NC_NAME] = '\0';
		        }
		if ((yylval = lookup(yytext)) == NULL) {
			yylval = install(yytext);
			}
		return (IDENT);
		}

\n		{
		lineno++ ;
		}
[+-]?[0-9]*\.[0-9]*([eE][+-]?[0-9]+)?[LlDd]? {
                double_val = atof(yytext);
                return (DOUBLE_CONST);
                }
[+-]?[0-9]*\.[0-9]*([eE][+-]?[0-9]+)?[Ff] {
                float_val = (float) atof(yytext);
                return (FLOAT_CONST);
                }
[+-]?[0-9]+[sS]|0[xX][0-9a-fA-F]+[sS] {
		short_val = strtol(yytext,(char**)0,0);
		return (SHORT_CONST);
	        }
[+-]?[0-9]+[lL]?|0[xX][0-9a-fA-F]+[lL]? {
		long_val = strtol(yytext,(char**)0,0);
#ifdef VMS  /* work around bug in VMS strtol() */
		if (STREQ(yytext, "-2147483648"))
		   long_val = -2147483648;
#endif /* VMS */
		return (LONG_CONST);
	        }
\'[^\\]\'          {
	        (void) sscanf(&yytext[1],"%c",&byte_val);
		return (BYTE_CONST);
                }
\'\\[0-7][0-7]?[0-7]?\'  {
		byte_val = strtol(&yytext[2], (char **) 0, 8);
		return (BYTE_CONST);
                }
\'\\x[0-9a-fA-F][0-9a-fA-F]?\'  {
		byte_val = strtol(&yytext[2], (char **) 0, 16);
		return (BYTE_CONST);
                }
\'\\.\'        {
	       switch (yytext[2]) {
	          case 'a': byte_val = '\a'; break;
     	          case 'b': byte_val = '\b'; break;
		  case 'f': byte_val = '\f'; break;
		  case 'n': byte_val = '\n'; break;
		  case 'r': byte_val = '\r'; break;
		  case 't': byte_val = '\t'; break;
		  case 'v': byte_val = '\v'; break;
		  case '\\': byte_val = '\\'; break;
		  case '?': byte_val = '\?'; break;
		  case '\'': byte_val = '\''; break;
		  default: byte_val = yytext[2];
	           }
		return (BYTE_CONST);
                }

[ \t\14\f]+	{/* whitespace */ ;
		}
.		return (yytext[0]) ;
