/*
 *	Copyright 1987, University Corporation for Atmospheric Research
 *	    Not for Direct Resale. All copies to include this notice.
 */
/*	$Id: cdf.c,v 1.28 89/12/07 18:17:11 davis Exp $ */

#include	"local_nc.h"
#include	"alloc.h"


void
NC_free_cdf(handle)
NC *handle ;
{
	extern void NC_free_array() ;

	if(handle == NULL)
		return ;
	NC_free_array(handle->dims) ;
	NC_free_array(handle->attrs) ;
	NC_free_array(handle->vars) ;

	xdr_destroy(handle->xdrs) ;
	m_free(handle->xdrs) ;
	fclose(handle->stream) ;

	m_free(handle) ;
}


NC *
NC_new_cdf(stream, mode)
FILE *stream ;
int mode ;
{
	NC *cdf ;
	enum xdr_op op ;

	cdf = (NC *)m_alloc(sizeof(NC)) ;
	if( cdf == NULL )
	{
		nc_serror("NC_new_cdf") ;
		return(NULL) ;
	}

	cdf->flags = mode ;
	cdf->stream = stream ;

	cdf->xdrs = (XDR *)m_alloc(sizeof(XDR)) ;
	if( cdf->xdrs == NULL)
	{
		nc_serror("NC_new_cdf: xdrs") ;
		fclose(cdf->stream) ;
		m_free(cdf) ;
		return(NULL) ;
	} /* else */	

	if( mode & NC_CREAT )
	{
		op = XDR_ENCODE ;
	} else {
		op = XDR_DECODE ;
	}
	xdrstdio_create( cdf->xdrs, cdf->stream, op ) ; /* (void) */

	cdf->dims = NULL ;
	cdf->attrs = NULL ;
	cdf->vars = NULL ;
	cdf->begin_rec = 0 ;
	cdf->recsize = 0 ;
	cdf->numrecs = 0 ;

	if(op == XDR_DECODE) /* Not NC_CREAT */
	{
		if(!xdr_cdf(cdf->xdrs, &cdf) )
		{
			nc_serror("NC_new_cdf: xdr_cdf") ;
			NC_free_cdf(cdf) ;
			return(NULL) ;
		}
		if( NC_computeshapes(cdf) == -1)
			return(NULL) ;
	}

	return(cdf) ;
}


/* 
 * Duplicate a description structure.
 * Can only be called for 'old' extant on disk, eg, old in DATA mode.
 */
NC *
NC_dup_cdf(stream, old)
FILE *stream ;
NC *old ;
{
	NC *cdf ;
	enum xdr_op op ;

	cdf = (NC *)m_alloc(sizeof(NC)) ;
	if( cdf == NULL )
	{
		nc_serror("NC_dup_cdf") ;
		return(NULL) ;
	}

	cdf->flags = old->flags | NC_INDEF ;
	cdf->stream = stream ;

	cdf->xdrs = (XDR *)m_alloc(sizeof(XDR)) ;
	if( cdf->xdrs == NULL)
	{
		nc_serror("NC_dup_cdf: xdrs") ;
		fclose(cdf->stream) ;
		m_free(cdf) ;
		return(NULL) ;
	} /* else */	

	cdf->dims = NULL ;
	cdf->attrs = NULL ;
	cdf->vars = NULL ;
	cdf->begin_rec = 0 ;
	cdf->recsize = 0 ;
	cdf->numrecs = 0 ;

	op = XDR_ENCODE ;

	xdrstdio_create( cdf->xdrs, cdf->stream, op ) ; /* (void) */

	old->xdrs->x_op = XDR_DECODE ;
	if(!xdr_cdf(old->xdrs, &cdf) )
	{
		nc_serror("NC_dup_cdf: xdr_cdf") ;
		NC_free_cdf(cdf) ;
		return(NULL) ;
	}
	if( NC_computeshapes(cdf) == -1)
		return(NULL) ;

	return(cdf) ;
}


ncinquire(cdfid, ndimsp, nvarsp, nattrsp, xtendimp)
int cdfid ;
int *ndimsp ;
int *nvarsp ;
int *nattrsp ;
int *xtendimp ;
{
	NC *handle ;
	extern NC *NC_check_id() ;

	cdf_routine_name = "ncinquire" ;

	handle = NC_check_id(cdfid) ;
	if(handle == NULL)
		return(-1) ;
	
	*nvarsp = (handle->vars != NULL) ? handle->vars->count : 0 ;
	*nattrsp = (handle->attrs != NULL) ? handle->attrs->count : 0 ;
	if(handle->dims != NULL)
	{
		NC_dim **dp ;
		int ii ;

		*ndimsp = handle->dims->count ;
		*xtendimp = -1 ;

		dp = (NC_dim**)handle->dims->values ;
		for(ii = 0 ; ii < handle->dims->count ; ii++, dp++)
		{
			if((*dp)->size == NC_UNLIMITED)
			{
				*xtendimp = ii ;
			}
		}
	} else {
		*ndimsp = 0 ;
		*xtendimp = -1 ;
	}

	return(cdfid) ;
}


bool_t
xdr_cdf(xdrs, handlep)
	XDR *xdrs;
	NC **handlep;
{

	u_long	magic = NCMAGIC ;

	if( xdrs->x_op == XDR_FREE)
	{
		NC_free_cdf(*handlep) ;
		return(TRUE) ;
	}
	
	if( xdr_getpos(xdrs) != 0)
	{
		if( !xdr_setpos(xdrs, 0) )
		{
			nc_serror("Can't set position to begin") ;
			return(FALSE) ;
		}
	}

	/* magic number */
	if( !xdr_u_long(xdrs, &magic) )
	{
		NCadvise(NC_EXDR, "xdr_u_long") ;
		return(FALSE) ;
	}

	if( xdrs->x_op == XDR_DECODE && magic != NCMAGIC )
	{
		if(magic == NCLINKMAGIC)
		{
			NCadvise(NC_NOERR, "link file not handled yet") ;
			return(FALSE) ;
		} /* else */
		NCadvise(NC_ENOTNC, "Not a netcdf file") ;
		return(FALSE) ;
	}

	if( !xdr_u_long(xdrs, &((*handlep)->numrecs)))
	{
		NCadvise(NC_EXDR, "xdr_u_long") ;
		return(FALSE) ;
	}
	if( !xdr_NC_array(xdrs, &((*handlep)->dims)))
	{
		NCadvise(NC_EXDR, "xdr_cdf:dims") ;
		return(FALSE) ;
	}
	if( !xdr_NC_array(xdrs, &((*handlep)->attrs)))
	{
		NCadvise(NC_EXDR, "xdr_cdf:attrs") ;
		return(FALSE) ;
	}
	if( !xdr_NC_array(xdrs, &((*handlep)->vars)))
	{
		NCadvise(NC_EXDR, "xdr_cdf:vars") ;
		return(FALSE) ;
	}	
	return(TRUE) ;
}


/*
 * How much space will the xdr'd NC description take.
 *
 */
NC_xlen_cdf(cdf)
NC *cdf ;
{
	int len = 8 ;

	if(cdf == NULL)
		return(0) ;

	len += NC_xlen_array(cdf->dims) ;
	len += NC_xlen_array(cdf->attrs) ;
	len += NC_xlen_array(cdf->vars) ;

	return(len) ;
}


#define RECPOS	4 	/* seek index of numrecs value */
bool_t
xdr_numrecs(xdrs, handle)
	XDR *xdrs;
	NC *handle;
{
	if( !xdr_setpos(xdrs, RECPOS) )
	{
		nc_serror("Can't set position to RECPOS") ;
		return(FALSE) ;
	}
	return(xdr_u_long(xdrs, &(handle->numrecs))) ;
}


bool_t
xdr_NC_fill(xdrs, type, alen)
XDR *xdrs ;
nc_type type ;
u_long alen ;
{
	u_char fillp[2*sizeof(double)] ;
	bool_t stat ;
	bool_t (*xdr_NC_fnct)() ;
	extern void NC_arrayfill() ;

		/* set up fill value */
		NC_arrayfill((Void *)fillp, (Size_t)2*sizeof(double),
			type) ;

		switch(type){
		case NC_BYTE :
		case NC_CHAR :
			alen /= 4 ;
			xdr_NC_fnct = xdrs->x_ops->x_putlong ;
			break ;
		case NC_SHORT :
#ifndef BIG_SHORTS
			alen /= 4 ;
			xdr_NC_fnct = xdrs->x_ops->x_putlong ;
			break ;
#else
			
			for(stat = TRUE ; stat && (alen > 0) ; alen -= 4)
			{
				stat = xdr_shorts(xdrs,fillp, 2) ;	
			}
			goto out ;
#endif
		case NC_LONG :
			alen /= 4 ;
			xdr_NC_fnct = xdrs->x_ops->x_putlong ;
			break ;	
		case NC_FLOAT :
			alen /= 4 ;
			xdr_NC_fnct = xdr_float ;
			break ;
		case NC_DOUBLE : 
			alen /= 8 ;
			xdr_NC_fnct = xdr_double ;
			break ;
		default :
			NCadvise(NC_EBADTYPE, "bad type %d", type) ;
			return(FALSE) ;
		}

		/* write out fill values */
		for(stat = TRUE ; stat && (alen > 0) ; alen--)
		{
			stat = (*xdr_NC_fnct)(xdrs,fillp) ;	
		}

		if(!stat)
		{
			NCadvise(NC_EXDR, "xdr_NC_fill") ;
			return(FALSE) ;
		}
		
		return(TRUE) ;
}
