/*
 *	Copyright 1988, University Corporation for Atmospheric Research
 *	    Not for Direct Resale. All copies to include this notice.
 */
#ifndef	lint
static char *rcsid[] = {
	"$Id: error.c,v 1.13 89/12/07 18:17:21 davis Exp $" ,
} ;
#endif	/* lint */


/* 
 *   Utility Functions to implement consistant error logging
 * mechanisms for netcdf
 *   Specification requires that this work on VMS and Sun OS(unix).
 * Tested on ultrix as well.
 */

/*LINTLIBRARY*/

#include	"local_nc.h"
#include	<stdio.h>
#ifdef STDC_ARGS
#include	<stdarg.h>
#else
#include	<varargs.h>
#endif /* STDC_ARGS */
#include <errno.h>
#ifdef ERRNO_MISSING
extern int errno;
#endif


int ncerr = NC_NOERR ;

/*
 * russ's last minute whistles
 *	These subroutines emit no messages unless NC_VERBOSE bit is on.
 *	These subroutines call exit() when NC_FATAL bit is on.
 */
int ncopts = (NC_FATAL | NC_VERBOSE) ;


/*
 *	Set to the the name of the current interface routine by the
 * interface routine.
 */
char *cdf_routine_name = "netcdf";


#ifdef vms
/*
 * On the vms system, when a system error occurs which is not
 * mapped into the unix styled errno values, errno is set EVMSERR
 * and a VMS error code is set in vaxc$errno.
 * This routine prints the systems message associated with status return
 * from a system services call.
 */

#include <descrip.h>
#include <ssdef.h>

static int
vmserror1( int status )
{
	short msglen ;
	char msgbuf[256] ;
	$DESCRIPTOR(message, msgbuf) ;
	register ret ;

	ret = SYS$GETMSG(status, &msglen, &message, 15, 0) ;
	
	switch(ret) {
	case SS$_BUFFEROVF :
	case SS$_NORMAL :
		(void) fprintf(stderr, "%.*s\n", msglen, msgbuf) ;
		break ;
	default :
		break ;
	}
	return(ret) ;
}
#endif /* vms */


/*
 * Log SYSTEM errors
 * Use where you would want to call perror(3).
 * Calling sequence is
 *	nc_serror(format, arg1, arg2,...)
 * with zero or more args of types compatible with the associated format
 * specifiers.  For example:
 *         nc_serror("shutting down");
 *	   nc_serror("can't open %s", file_name);
 *         nc_serror("process %d in state %s",pid,state);
 */
#ifdef STDC_ARGS
void
nc_serror(char *fmt, ...)
#else
/*VARARGS1*/
void
nc_serror(fmt, va_alist)
     char *fmt ;
     va_dcl
#endif /* STDC_ARGS */
{
    va_list args ;
    int errnum ;
	static char unknown[] = "Unknown Error";

#ifdef STDC_STRERROR
    char * cp;
    extern char *strerror();
#else
    extern int sys_nerr;
    extern char *sys_errlist[];
#endif /* STDC_STRERROR */

    errnum = errno;		/* save real errno in case we wipe it out */

	if( ncopts & NC_VERBOSE )
	{
#ifdef STDC_ARGS
		va_start(args, fmt) ;
#else
		va_start(args) ;
#endif /* STDC_ARGS */
		(void) fprintf(stderr,"%s: ", cdf_routine_name);
		(void) vfprintf(stderr,fmt,args) ;
		va_end(args) ;
	} /* */

    switch(errnum) {
	case 0 :
		ncerr = NC_NOERR ;
		if( ncopts & NC_VERBOSE )
			(void) fputc('\n',stderr) ;
		break ;
#ifdef vms
	case EVMSERR :
		ncerr = NC_SYSERR ;
		if( ncopts & NC_VERBOSE )
		{
			(void) fputc(': ',stderr) ;
			(void) vmserror1(vaxc$errno) ;
		}
		break ;
#endif /* vms */
    default :
		ncerr = NC_SYSERR ;
		if( ncopts & NC_VERBOSE )
		{
#ifdef STDC_STRERROR
        (void) fprintf(stderr,": %s\n",
        	(cp = strerror(errnum)) == NULL ? unknown : cp ) ;
#else
        (void) fprintf(stderr,": %s\n",
			errnum < sys_nerr ? sys_errlist[errnum] : unknown );
#endif /* STDC_STRERROR */
		}
		break ;
    }
	if( ncopts & NC_VERBOSE )
    	(void) fflush(stderr);	/* to ensure log files are current */

	if( ncopts & NC_FATAL )
	{
		exit(ncopts) ;
	}
	/* else */

    errno = 0 ;
}


/*
 * Like nc_serror above, but doesn't check for system error.
 * Use for logging error conditions which are not system errors.
 * Calling sequence is
 *	NCadvise(ncerr, format, arg1, arg2,...)
 * with zero or more args of types compatible with the associated format
 * specifiers.  For example:
 *         NCadvise(NC_NOERR, "just advice");
 *         NCadvise(NC_EBADID, "%d is not a valid cdf id", cdfid);
 */
#ifdef STDC_ARGS
void
NCadvise(int err, char *fmt,...)
#else
/*VARARGS1*/
void
NCadvise(err, fmt, va_alist)
     int err ;
     char *fmt ;
     va_dcl
#endif /* STDC_ARGS */
{
	va_list args ;

	ncerr = err ;

	if( ncopts & NC_VERBOSE )
	{
		(void) fprintf(stderr,"%s: ", cdf_routine_name);
#ifdef STDC_ARGS
		va_start(args ,fmt) ;
#else
		va_start(args) ;
#endif /* STDC_ARGS */
		(void) vfprintf(stderr,fmt,args) ;
		va_end(args) ;
		(void) fputc('\n',stderr) ;
		(void) fflush(stderr);	/* to ensure log files are current */
	}

	if( (ncopts & NC_FATAL) && ncerr != NC_NOERR )
	{
		exit(ncopts) ;
	}
}
