/*
 *	Copyright 1988, University Corporation for Atmospheric Research
 *	    Not for Direct Resale. All copies to include this notice.
 */
/*	$Id: file.c,v 1.32 89/12/18 11:15:47 mschmidt Exp Locker: davis $ */

/* needed for 'access' function in NC_open */
#if defined(unix) && !defined(i386) && !defined(cray) /* unicos has fouled up includes */
#include	<sys/types.h>
#include	<sys/file.h>
#endif
#ifndef F_OK
#define F_OK	0
#endif

#ifdef MSDOS
#include	<fcntl.h>	/* O_BINARY */
#endif

#ifdef DEBUG
#include <assert.h>
#endif /* DEBUG */

#include	<string.h>
#include	"local_nc.h"
#include	"alloc.h"

#ifndef USE_BFLAG 	/* Doesn't Understand the "b" (binary flag to fopen) */
#define CREAT_FOPENFLAGS "w+"
#define WRITE_FOPENFLAGS "r+"
#define NOWRITE_FOPENFLAGS "r"
#else
#define CREAT_FOPENFLAGS "w+b"
#define WRITE_FOPENFLAGS "r+b"
#define NOWRITE_FOPENFLAGS "rb"
#endif /* !USE_BFLAG */

static int _ncdf = 0 ; /* number of open cdf's */
static NC *_cdfs[MAX_NC_OPEN+1] ;
/* _cdfs[MAX_NC_OPEN] is reserved for stash during redefinition */
#define OLD (_cdfs[MAX_NC_OPEN])

#if !defined(__STDC__) && defined(unix)
#define remove(fpath) unlink((fpath))
#endif

extern void NC_free_cdf() ;

#if defined(cray) || defined(i386)
/* UNICOS ain't got this nor does system V Rel 2 */
static int
rename( from, to)
char *from ;
char *to ;
{
#ifdef DEBUGC
	fprintf(stderr, " from %s, to %s\n", from, to) ;
#endif
	unlink(to) ;
	if( link( from, to) != -1 && unlink(from) != -1)
		return(0) ;
	return(-1) ;
}
#endif

/*
 *  Check validity of cdf handle, return pointer to NC struct or
 * NULL on error.
 */
NC *
NC_check_id(cdfid)
int cdfid ;
{
	NC *handle ;

	handle = ( cdfid >= 0 && cdfid < _ncdf) ? _cdfs[cdfid] : NULL ;
	if(handle == NULL)
	{
		NCadvise(NC_EBADID, "%d is not a valid cdfid", cdfid) ;
		return(NULL) ;
	}
	return(handle) ;
}


/*
 *  Check to see if in define mode.
 * If 'iserr' arg is true, advise.
 */
bool_t
NC_indefine(cdfid, iserr) /* Should be a Macro ? */
int cdfid ;
bool_t iserr ;
{
	bool_t ret  ;
	ret = (cdfid >= 0 && cdfid < _ncdf) ?
		(bool_t)(_cdfs[cdfid]->flags & NC_INDEF) : FALSE ;
	if(!ret && iserr)
	{
		NCadvise(NC_ENOTINDEFINE, "Not in define mode") ;
	}
	return(ret) ;
}


/*
 *  Common code for ncopen and nccreate.
 */
static int
NC_open(path, mode )
char	*path ;	/* file name */
int mode ;
{
	extern NC *NC_new_cdf() ;
	char *fmode ;
	FILE *fp ;
	NC *handle ;
	int id ;

#ifdef MSDOS
	/*
 	 * set default mode to binary to suppress the expansion of
	 * 0x0f into CRLF
	 */
	if(_fmode != O_BINARY)
		_fmode = O_BINARY ;
#endif

	/* find first available id */
	for(id = 0 ; id < _ncdf; id++)
		if( _cdfs[id] == NULL) break ;

	if(id == _ncdf && _ncdf >= MAX_NC_OPEN) /* will need a new one */
	{
		NCadvise(NC_ENFILE, "maximum number of open cdfs %d exceeded",
			 _ncdf) ;
		return(-1) ;
	}

	/* should fmodes have 'b' appended? */
	switch(mode & 0x0f) {
	case NC_NOCLOBBER :
		if( access(path, F_OK) != -1 )
		{
			NCadvise(NC_EEXIST, "\"%s\": File exists", path) ;
			return(-1) ;
		}
		/* fall into */
	case NC_CLOBBER :
		fmode = CREAT_FOPENFLAGS ;
		break ;
	case NC_WRITE :
		fmode = WRITE_FOPENFLAGS ;
		break ;
	case NC_NOWRITE :
		fmode = NOWRITE_FOPENFLAGS ;
		break ;
	default:
		NCadvise(NC_EINVAL, "Bad flag %0x", mode & 0x0f) ;
		return(-1) ;
	}

#ifndef vms
	fp = fopen(path, fmode) ;
#else
	fp = fopen(path, fmode,
		"mbf=2","mbc=16") ;
#endif /* !vms */
	if( fp == NULL )
	{
		nc_serror("filename \"%s\"", path) ;
		return(-1) ;
	} /* else */
	
	handle = NC_new_cdf(fp,mode) ;
	if( handle == NULL)
	{
		(void)fclose(fp) ;
		if((mode & 0x0f) == NC_CLOBBER)
		{
			if( remove(path) != 0 )
				nc_serror("couldn't remove filename \"%s\"", path) ;
		}
		return(-1) ;
	}

	(void) strncpy(handle->path, path, FILENAME_MAX) ;
	_cdfs[id] = handle ;
	if(id == _ncdf)
		_ncdf++ ;
	return(id) ;
}


nccreate(path, cmode)
char	*path ;	/* file name */
int 	cmode ;
{
	cdf_routine_name = "nccreate" ;

	if(cmode & NC_CREAT)
	{
		return(NC_open(path, cmode)) ;
	}
	NCadvise(NC_EINVAL, "Bad Flag") ;
	return(-1) ;
}


ncopen(path,mode)
char	*path ;	/* file name */
int 	mode ;
{
	cdf_routine_name = "ncopen" ;
	if(mode & NC_CREAT)
	{
		NCadvise(NC_EINVAL, "Bad Flag") ;
		return(-1) ;
	}
	return(NC_open(path, mode)) ;
}


ncsync(id)
int id ;
{
	NC *handle ;

	cdf_routine_name = "ncsync" ;

	handle = NC_check_id(id) ; 
	if(handle == NULL)
		return(-1) ;

	if( handle->flags & NC_INDEF )
	{
		NCadvise(NC_EINDEFINE, "Unfinished definition") ;
		return(-1) ;
	}

	(void)fflush((FILE *)handle->xdrs->x_private);

	if(handle->flags & NC_RDWR)
	{
		handle->xdrs->x_op = XDR_ENCODE ;
		if(handle->flags & NC_HDIRTY)
		{
			if(!xdr_cdf(handle->xdrs, &handle) )
				return(-1) ;
			handle->flags &= ~(NC_NDIRTY | NC_HDIRTY) ;
		}
		else if(handle->flags & NC_NDIRTY)
		{
			if(!xdr_numrecs(handle->xdrs, handle) )
				return(-1) ;
			handle->flags &= ~(NC_NDIRTY) ;
		}
	}

	return(0) ;
}


/*
 *  In data mode, same as ncclose ;
 * In define mode, restore previous definition ;
 * In create, remove the file ;
 */
ncabort(cdfid)
int cdfid ;
{
	NC *handle ;
	char path[FILENAME_MAX + 1] ;
	unsigned flags ;

	cdf_routine_name = "ncabort" ;


	handle = NC_check_id(cdfid) ; 
	if(handle == NULL)
		return(-1) ;

	flags = handle->flags ; /* need to save past free_cdf */

	/* NC_CREAT implies NC_INDEF, in both cases need to remove handle->path */
	if(flags & (NC_INDEF | NC_CREAT))
	{
		(void)strncpy(path, handle->path, FILENAME_MAX) ; /* stash path */
		if(!(flags & NC_CREAT)) /* redef */
		{
			NC_free_cdf(OLD) ;
			OLD = NULL ;
		}
	}
	else if(handle->flags & NC_RDWR)
	{
		handle->xdrs->x_op = XDR_ENCODE ;
		if(handle->flags & NC_HDIRTY)
		{
			if(!xdr_cdf(handle->xdrs, &handle) )
				return(-1) ;
		}
		else if(handle->flags & NC_NDIRTY)
		{
			if(!xdr_numrecs(handle->xdrs, handle) )
				return(-1) ;
		}
	}

	NC_free_cdf(handle) ; /* calls fclose */

	if(flags & (NC_INDEF | NC_CREAT))
	{
		if( remove(path) != 0 )
			nc_serror("couldn't remove filename \"%s\"", path) ;
	}

	_cdfs[cdfid] = NULL ;

	if(cdfid == _ncdf - 1)
		_ncdf-- ;

	return(0) ;
}


/* 
 * Why anyone would want to do this is beyond me
 * All it really does is assure you that you will use up
 * about 5 times as much system time.
 */
ncnobuf(cdfid)
int cdfid ;
{
	NC *handle ;
	char *fmode ;

	cdf_routine_name = "ncnobuf" ;

	handle = NC_check_id(cdfid) ; 
	if(handle == NULL)
		return(-1) ;
	
	if(handle->flags & NC_CREAT)
		fmode = CREAT_FOPENFLAGS ;
	else if(handle->flags & NC_RDWR)
		fmode = WRITE_FOPENFLAGS ;
	else
		fmode = NOWRITE_FOPENFLAGS ;

	xdr_destroy(handle->xdrs) ;

	handle->stream = freopen(handle->path, fmode, handle->stream) ; 
	if( handle->stream == NULL )
	{
		nc_serror("Couldn't reopen \"%s\"", handle->path) ;
		return(-1) ;
	}

	setbuf(handle->stream, NULL) ; /* the meat of the matter */
	/* should these options be orthogonal to the setbuf ?*/
	handle->flags |= (NC_NSYNC | NC_HSYNC) ;

	/*
	 * use of handle->xdrs->x_op is legal below since the 'destroy'
	 * above only affects the stdio buffer
	 */
	xdrstdio_create( handle->xdrs, handle->stream, handle->xdrs->x_op ) ;

	return(0);
}


ncredef(cdfid)
int cdfid ;
{
	NC *handle ;
	NC *new ;
	char *scratchfile ;
	FILE *fp ;
	extern NC *NC_dup_cdf() ;
#ifdef MSDOS
	extern char *getcwd() ;
#endif

	cdf_routine_name = "ncredef" ;

	handle = NC_check_id(cdfid) ; 
	if(handle == NULL)
		return(-1) ;
	if( handle->flags & NC_INDEF) /* in define mode already */
	{
		NCadvise(NC_EINDEFINE, "%s: in define mode aleady", OLD->path) ;
		return(-1) ;
	}
	if(!(handle->flags & NC_RDWR))
	{
		NCadvise(NC_EPERM, "%s: NC_NOWRITE", handle->path) ;
		return(-1) ;
	}
	(void)fflush((FILE *)handle->xdrs->x_private);


/*
 * unicos ignores the first arg to tempnam
 * always seems to want to put the file in another file system
 */
#if !defined(cray) && !defined(NeXT)
#if defined(unix) || defined(MSDOS)
	scratchfile = tempnam(getcwd(NULL,FILENAME_MAX),"nc.") ;
#else
	scratchfile = tmpnam(NULL) ;
#endif /* unix & !cray */
#else /* !cray */
	scratchfile = m_alloc(FILENAME_MAX +1) ;
#endif /* !cray */
	if(scratchfile == NULL)
	{
		nc_serror("Couldn't get tmpnam") ;
		return(-1) ;
	}
#if defined(cray) || defined(NeXT) /* synthesize a name ourselves */
	strncpy(scratchfile, handle->path, FILENAME_MAX-4) ;
	strcat(scratchfile, ".tmp") ;
#endif

#ifndef vms
	fp = fopen(scratchfile, CREAT_FOPENFLAGS) ;
#else
	fp = fopen(scratchfile, CREAT_FOPENFLAGS,
		"mbf=2","mbc=16") ;
#endif /* !vms */
	if( fp == NULL )
	{
		nc_serror("filename \"%s\"", scratchfile) ;
#ifdef unix
		m_free(scratchfile) ;
#endif /* unix */
		return(-1) ;
	} /* else */

	new = NC_dup_cdf(fp, handle) ;
	if(new == NULL)
	{
#ifdef unix
		m_free(scratchfile) ;
#endif
		return(-1) ;
	}

	handle->flags |= NC_INDEF ;
	(void) strncpy(new->path, scratchfile, FILENAME_MAX) ;
#ifdef unix
	m_free(scratchfile) ;
#endif

	_cdfs[cdfid] = new ;
	_cdfs[MAX_NC_OPEN] = handle ;

	return(0) ;
}


/*
 * Compute offsets and put into the header
 */
static void
NC_begins(handle)
NC *handle ;
{
	int ii ;
	u_long index = 0 ;
	NC_var **vpp ;
	NC_var *last = NULL ;

	if(handle->vars == NULL) 
		return ;

	index = NC_xlen_cdf(handle) ;

	/* loop thru vars, first pass is for the 'non-record' vars */
	vpp = (NC_var **)handle->vars->values ;
	for(ii = 0 ; ii < handle->vars->count ; ii++, vpp++)
	{
		if( IS_RECVAR(*vpp) )
		{
			continue ;	/* skip record variables on this pass */
		}

		(*vpp)->begin = index ;
		index += (*vpp)->len ;
#ifdef EDEBUG
		NCadvise(NC_NOERR, "%s pass 1 begin %d, length %d",
			(*vpp)->name->values, (*vpp)->begin, (*vpp)->len) ;
#endif /* EDEBUG */
	}

	handle->begin_rec = index ;
	handle->recsize = 0 ;

	/* loop thru vars, second pass is for the 'non-record' vars */
	vpp = (NC_var **)handle->vars->values ;
	for(ii = 0 ; ii < handle->vars->count ; ii++, vpp++)
	{
		if( !IS_RECVAR(*vpp) )
		{
			continue ;	/* skip non-record variables on this pass */
		}

		(*vpp)->begin = index ;
#ifdef EDEBUG
		NCadvise(NC_NOERR, "%s pass 2 begin %d, len %d, *dsizes %d",
			(*vpp)->name->values, (*vpp)->begin, index, (*vpp)->len, *(*vpp)->dsizes ) ;
#endif /* EDEBUG */
		index += (*vpp)->len ;
		handle->recsize += (*vpp)->len ;
		last = (*vpp) ;
	}
	/*
	 * for special case of exactly one record variable, pack values
	 */
	if(last != NULL && handle->recsize == last->len)
		handle->recsize = *last->dsizes ;
	handle->numrecs = 0 ;
}


/*
 * Copy len bytes from source to target.
 * Streams target and source should be positioned before the call.
 */
static bool_t
NC_dcpy( target, source, len)
XDR *target ;
XDR *source ;
Size_t len ;
{
	Void *buf ;
	bool_t stat ;
	buf = (Void *)m_alloc(len) ;
	if(buf == NULL)
	{
		nc_serror("dcpy") ;
		stat = FALSE ;
	}
	else if( (stat = XDR_GETBYTES(source, buf, len)))
	{
		if( !(stat = XDR_PUTBYTES(target, buf, len)))
		{
			NCadvise(NC_EXDR, "NC_dcpy: XDR_PUTBYTES") ;
		}
	}else {
		NCadvise(NC_EXDR, "NC_dcpy: XDR_GETBYTES") ;
	}
	m_free(buf) ;
	return(stat) ;
}


/*
 * XDR the data of varid in old, target is the new xdr stream
 */
static bool_t
NC_vcpy( target, old, varid)
XDR *target ;
NC *old ;
int varid ;
{
	NC_var **vpp ;
	vpp = (NC_var **)old->vars->values ;
	vpp += varid ;

	if( !xdr_setpos(old->xdrs, (*vpp)->begin) )
	{
		NCadvise(NC_EXDR, "NC_vcpy: xdr_setpos") ;
		return(FALSE) ;
	}
		
	return(NC_dcpy(target, old->xdrs, (Size_t)(*vpp)->len)) ;
}


/*
 * XDR the data of (varid, recnum) in old, target is the new xdr stream
 */
static bool_t
NC_reccpy( target, old, varid, recnum)
XDR *target ;
NC *old ;
int varid ;
int recnum ;
{
	NC_var **vpp ;
	vpp = (NC_var **)old->vars->values ;
	vpp += varid ;

	if( !xdr_setpos(old->xdrs,
			(*vpp)->begin + old->recsize*recnum )){
		NCadvise(NC_EXDR, "NC_reccpy: xdr_setpos") ;
		return(FALSE) ;
	}
	

	return(NC_dcpy(target, old->xdrs, (Size_t)(*vpp)->len)) ;
}


/*
 *  Common code for ncendef, ncclose(endef)
 */
static
NC_endef( cdfid, handle )
int cdfid ;
NC *handle ;
{
	XDR *xdrs ;
	int ii ;
	int jj = 0 ;
	NC_var **vpp ;

	NC_begins(handle) ;

	xdrs = handle->xdrs ;
	xdrs->x_op = XDR_ENCODE ;

	if(!xdr_cdf(xdrs, &handle) )
	{
		nc_serror("xdr_cdf") ;
		return(-1) ;
	}

	if(handle->vars == NULL) 
		goto done ;
	
	/* loop thru vars, first pass is for the 'non-record' vars */
	vpp = (NC_var **)handle->vars->values ;
	for(ii = 0 ; ii < handle->vars->count ; ii++, vpp++)
	{
		if( IS_RECVAR(*vpp) )
		{
			continue ;	/* skip record variables on this pass */
		}

#ifdef DEBUG
		assert( (*vpp)->begin == xdr_getpos(xdrs) ) ;
#endif /* DEBUG */

		if( !(handle->flags & NC_CREAT) && OLD->vars != NULL
			&& ii < OLD->vars->count)
		{
			/* copy data */
			if( !NC_vcpy(xdrs, OLD, ii) )
				return(-1) ;
			continue ;
		} /* else */

		if( !xdr_NC_fill(xdrs, (*vpp)->type, (*vpp)->len) )
			return(-1) ;
	}

	if(!(handle->flags & NC_CREAT)) /* after redefinition */
	{
		for(jj = 0 ; jj < OLD->numrecs ; jj++)
		{
			vpp = (NC_var **)handle->vars->values ;
			for(ii = 0 ; ii < handle->vars->count ; ii++, vpp++)
			{
				if( !IS_RECVAR(*vpp) )
				{
					continue ;	/* skip non-record variables on this pass */
				}
				if( OLD->vars != NULL && ii < OLD->vars->count)
				{
					/* copy data */
					if( !NC_reccpy(xdrs, OLD, ii, jj) )
						return(-1) ;
					continue ;
				} /* else */
				if( !xdr_NC_fill(xdrs, (*vpp)->type, (*vpp)->len) )
					return(-1) ;
			}
		}
		handle->numrecs = OLD->numrecs ;
		if(!xdr_numrecs(handle->xdrs, handle) )
			return(-1) ;
	}

#ifdef EDEBUG
	NCadvise(NC_NOERR, "begin %d, recsize %d, numrecs %d",
		handle->begin_rec, handle->recsize, handle->numrecs) ;
#endif /* EDEBUG */

	if(!(handle->flags & NC_CREAT)) /* redefine */
	{
#if !defined(unix) && !defined(vms)
		/* pc style file system, macOS */
		xdr_destroy(OLD->xdrs) ;
		fclose(OLD->stream) ;
		OLD->stream = NULL ;
		OLD->xdrs->x_private = NULL ; /* xdr implementation dependent */
		if(remove(OLD->path) != 0)
		{
			nc_serror("remove of %s failed", OLD->path) ;
		}
#endif
#ifdef MSDOS
                xdr_destroy(handle->xdrs) ;
                fclose(handle->stream) ;
#endif
		if( rename(handle->path, OLD->path) != 0)
		{
			nc_serror("rename failed") ;
			/* try to restore state prior to redef */
			NC_free_cdf(handle) ;
			_cdfs[cdfid] = OLD ;
			OLD = NULL ;
			handle->flags &= NC_RDWR ;
			return(-1) ;
		}
		(void) strncpy(handle->path, OLD->path, FILENAME_MAX) ;
#ifdef MSDOS
                handle->stream = fopen(handle->path, WRITE_FOPENFLAGS) ;
                if( handle->stream == NULL )
                {
                        nc_serror("reopen \"%s\"", handle->path) ;
                        return(-1) ;
                }
                xdrstdio_create( handle->xdrs, handle->stream,
                        handle->xdrs->x_op ) ;
#endif
		NC_free_cdf(OLD) ;
		OLD = NULL ;
	}

done:
	handle->flags &= ~(NC_CREAT | NC_INDEF | NC_NDIRTY | NC_HDIRTY) ;
	return(0) ;
}


ncendef( cdfid )
int cdfid ;
{
	NC *handle ;

	cdf_routine_name = "ncendef" ;

	handle = NC_check_id(cdfid) ; 
	if(handle == NULL)
		return(-1) ;
	if( !NC_indefine(cdfid,TRUE) )
		return(-1) ;
	return( NC_endef(cdfid, handle) ) ;
}


ncclose( cdfid )
int cdfid ;
{
	NC *handle ;

	cdf_routine_name = "ncclose" ;

	handle = NC_check_id(cdfid) ; 
	if(handle == NULL)
		return(-1) ;

	if( handle->flags & NC_INDEF)
	{
		if( NC_endef(cdfid, handle) == -1 ) ;
		{
			return( ncabort(cdfid) ) ;
		}
	}
	else if(handle->flags & NC_RDWR)
	{
		handle->xdrs->x_op = XDR_ENCODE ;
		if(handle->flags & NC_HDIRTY)
		{
			if(!xdr_cdf(handle->xdrs, &handle) )
				return(-1) ;
		}
		else if(handle->flags & NC_NDIRTY)
		{
			if(!xdr_numrecs(handle->xdrs, handle) )
				return(-1) ;
		}
	}

	NC_free_cdf(handle) ; /* calls fclose */

	_cdfs[cdfid] = NULL ;

	if(cdfid == _ncdf - 1)
		_ncdf-- ;

	return(0) ;
}
