/*
 *	Copyright 1988, University Corporation for Atmospheric Research
 *	    Not for Direct Resale. All copies to include this notice.
 */
/*	$Id: string.c,v 1.19 89/12/07 18:31:54 davis Exp $ */

#include	"local_nc.h"
#include	"alloc.h"

#include	<string.h>
#ifndef STDC_INCLUDES
/* memxxx() functions aren't in <string.h> */
#include	<memory.h>
#endif /* !STDC_INCLUDES */

NC_string *
NC_new_string(count, str)
unsigned count ;
char *str ;
{
	NC_string *ret ;
	Size_t memlen ;

	ret = (NC_string *)m_alloc(sizeof(NC_string)) ;
	if( ret == NULL )
		goto alloc_err ;
	ret->count = count ;
	if(count != 0 ) /* allocate */
	{
		memlen = count + 1 ;
		ret->values = m_alloc(memlen) ;
		if(ret->values == NULL)
			goto alloc_err ;
		if(str != NULL)
		{
			(void)strncpy(ret->values, str, count) ;
			ret->values[count] = 0 ;
		}
	} else { /* use what what you were given */
		ret->values = NULL ;
	}
	
	return(ret) ;
alloc_err :
	nc_serror("NC_new_string") ;
	return(NULL) ;
}


/*
 * Free string, and, if needed, its values.
 */
void
NC_free_string(cdfstr)
NC_string *cdfstr ;
{
	if(cdfstr==NULL)
		return ;
	if(cdfstr->values != NULL)
		m_free(cdfstr->values) ;
	m_free(cdfstr) ;
}


NC_string *
NC_re_string(old, count, str)
NC_string *old ;
unsigned count ;
char *str ;
{
	if(old->count < count) /* punt */
	{
		NCadvise(NC_ENOTINDEFINE,
			"Must be in define mode to increase name length %d",
			old->count) ;
		return(NULL) ;
	}
	(void)strncpy(old->values, str, count) ;
	(void)memset(old->values + count +1, 0, (int)old->count - (int)count +1) ;
	old->values[count] = 0 ;
	return(old) ;
}

bool_t
xdr_NC_string(xdrs, spp)
	XDR *xdrs;
	NC_string **spp;
{
	u_long count ;

	switch (xdrs->x_op) {
	case XDR_FREE:
		NC_free_string((*spp)) ;
		return(TRUE) ;
	case XDR_DECODE:
		/* need the length to pass to new */
		if (! xdr_u_long(xdrs, &count)) {
			return (FALSE);
		}
		if( count == 0)
		{
		   *spp = NULL ;
		   return(TRUE) ;
		} /* else */
		(*spp) = NC_new_string((unsigned)count, NULL) ;
		if((*spp) == NULL)
			return(FALSE) ;
		(*spp)->values[count] = 0 ;
		/* then deal with the characters */
		return (xdr_opaque(xdrs, (*spp)->values, (*spp)->count));
	case XDR_ENCODE:
		/* first deal with the length */
		if(*spp == NULL)
		{
			count = 0 ;
			return( xdr_u_long(xdrs, &count) ) ;
		} /* else */
		count = (*spp)->count ;
		if (! xdr_u_long(xdrs, &count) ) {
			return (FALSE);
		}
		/* then deal with the characters */
		return (xdr_opaque(xdrs, (*spp)->values, (*spp)->count));
	}
	return(FALSE) ;
}


/*
 * How much space will the xdr'd string take.
 *
 */
NC_xlen_string(cdfstr)
NC_string *cdfstr ;
{
	int len = 4 ;
	int rem ;
	if(cdfstr!=NULL)
	{
		len += cdfstr->count ;
		if( (rem = len%4) != 0)
			len += 4 - rem ;
	}
	return(len) ;
}
