/*
 *	Copyright 1988, University Corporation for Atmospheric Research
 *	    Not for Direct Resale. All copies to include this notice.
 */
/*	$Id: var.c,v 1.34 89/12/07 18:17:54 davis Exp $ */

#include	<string.h>
#include	"local_nc.h"
#include	"alloc.h"


NC_var *
NC_new_var(name,type,ndims,dims)
char *name ;
nc_type type ;
int ndims ;
int *dims ;
{
	NC_var *ret ;
	extern NC_string *NC_new_string() ;
	extern NC_iarray *NC_new_iarray() ;
	extern NC_array *NC_new_array() ;
	extern Size_t NC_typelen() ;

	ret = (NC_var *)m_alloc(sizeof(NC_var)) ;
	if( ret == NULL )
		goto alloc_err ;

	ret->name = NC_new_string((unsigned)strlen(name),name) ;
	if( ret->name == NULL)
		goto alloc_err ;

	ret->assoc = NC_new_iarray((unsigned)ndims, dims) ;
	if( ret->assoc == NULL)
		goto alloc_err ;

	ret->shape = NULL ;
	ret->dsizes = NULL ;
	ret->attrs = NULL ;
	ret->type = type ;
	ret->len = 0 ;
	ret->szof = NC_typelen(type) ;
	ret->begin = 0 ;

	return(ret) ;
alloc_err :
	nc_serror("NC_new_var") ;
	return(NULL) ;
}


/*
 * Free var
 */
void
NC_free_var(var)
NC_var *var ;
{
	extern void NC_free_string() ;
	extern void NC_free_iarray() ;
	extern void NC_free_array() ;

	if(var == NULL)
		return ;
	NC_free_string(var->name) ;
	NC_free_iarray(var->assoc) ;
	if(var->shape != NULL)
		m_free(var->shape) ;
	if(var->dsizes != NULL)
		m_free(var->dsizes) ;
	NC_free_array(var->attrs) ;
	m_free(var) ;
}


/*
 * 'compile' the shape and len of a variable
 *  return -1 on error
 */
static 
NC_var_shape(var, dims)
NC_var *var ;
NC_array *dims;
{
	unsigned *shape, *dsizes ;
	int ii ;
	unsigned *shp, *dsp, *op ;
	int *ip ;
	NC_dim **dp ;
	Size_t xszof ;

	xszof = NC_xtypelen(var->type) ;
	var->shape = NULL ;
	var->dsizes = NULL ;

	/*
	 * Allocate the shape array
	 */
	ii = var->assoc->count ;
	if(ii == 0)
	{
		var->len = xszof ;
		return(0) ;
	}
	shape = (unsigned *)m_alloc(ii * sizeof(unsigned)) ;
	if(shape == NULL)
	{
		nc_serror("NC_var_shape") ;
		return(-1) ;
	}

	/*
	 * use the user supplied dimension indices
	 * to determine the shape
	 */
	for(ip = var->assoc->values, op = shape
		; ii > 0 ; ii--)
	{
		if(*ip < 0 || *ip >= ((dims != NULL) ? dims->count : 1) )
		{
			NCadvise(NC_EBADDIM, "Bad dimension id %d", *ip) ;
			m_free(shape) ;
			return(-1) ;
		}
		dp = (NC_dim **)dims->values + *ip ;
		*op = (*dp)->size ;
		if(*op == NC_UNLIMITED && ii != var->assoc->count )
		{
			NCadvise(NC_EUNLIMPOS, "NC_UNLIMITED size applied to index other than 0 %d",
				var->assoc->count - ii) ;
			m_free(shape) ;
			return(-1) ;
		}
		op++ ; ip++ ;
	}
	var->shape = shape ;

	/*
	 * Allocate the dsizes array
	 */
	ii = var->assoc->count ;
	dsizes = (unsigned *)m_alloc(ii * sizeof(unsigned)) ;
	if(dsizes == NULL)
	{
		nc_serror("NC_var_dsizes") ;
		return(-1) ;
	}
	var->dsizes = dsizes ;

	/* 
	 * Compute var->len and the dsizes
	 */
	shp = shape + var->assoc->count - 1 ; /* count is > 0 here */
	dsp = dsizes + var->assoc->count - 1 ;
	var->len = (*shp) ? (*shp) : 1 ; /* boundary condition for rec */
	var->len *= xszof ;
	if(dsp != NULL) *dsp = xszof ;

	for( shp--, dsp-- ; shp >= shape ; shp--,dsp--)
	{
		*dsp = var->len ;
		if(shp != shape || *shp ) /* include last mult for non-rec vars */
			var->len *= *shp ;
	}

	switch(var->type) {
	case NC_BYTE :
	case NC_CHAR :
	case NC_SHORT :
		if( var->len%4 != 0 )
		{
			var->len += 4 - var->len%4 ; /* round up */
	/*		*dsp += 4 - *dsp%4 ; */
		}
	}

#if 0
	NCadvise(NC_NOERR, "%s var->len %d, var->szof %d",
		var->name->values, var->len, var->szof) ;
	arrayp("\tshape", var->assoc->count, var->shape) ;
	arrayp("\tdsizes", var->assoc->count, var->dsizes) ;
#endif
	return(var->assoc->count) ;
}


ncvardef(cdfid, name, type, ndims, dims)
int cdfid ;
char *name ;
nc_type type ;
int ndims ;
int dims[] ;
{
	NC *handle ;
	NC_var *var[1] ;
	NC_var **dp ;
	int ii ;
	int len ;
	extern NC *NC_check_id() ;
	extern NC_array *NC_new_array() ;
	extern Void *NC_incr_array() ;

	cdf_routine_name = "ncvardef" ;

	if( !NC_indefine(cdfid,TRUE) )
		return(-1) ;

	handle = NC_check_id(cdfid) ;
	if(handle == NULL)
		return(-1) ;

	if(!NCcktype(type))
		return(-1) ;

	if(ndims < 0) /* 0 => scalar ? */
	{
		NCadvise(NC_EINVAL, "Number of dimensions %d < 0", ndims) ;
		return(-1) ;
	} 

	if(ndims > 0 )
	{
		if(handle->dims == NULL || ndims > handle->dims->count )
		{
			NCadvise(NC_EINVAL, "Invalid number of dimensions %d > %d",
				ndims, (handle->dims != NULL) ? handle->dims->count : 0) ;
			return(-1) ;
		}
	}

	if(handle->vars == NULL) /* first time */
	{
		var[0] = NC_new_var(name,type,ndims,dims) ;
		if(var[0] == NULL)
			return(-1) ;
		handle->vars = NC_new_array(NC_VARIABLE,(unsigned)1,
			(Void *)var) ;
		if(handle->vars == NULL)
			return(-1) ;
	} else if(handle->vars->count >= MAX_NC_VARS)
	{
		NCadvise(NC_EMAXVARS, "maximum number of variables %d exceeded",
			handle->vars->count ) ;
		return(-1) ;
	} else {
		/* check for name in use */
		len = strlen(name) ;
		dp = (NC_var**)handle->vars->values ;
		for(ii = 0 ; ii < handle->vars->count ; ii++, dp++)
		{
			if( len == (*dp)->name->count &&
				strncmp(name, (*dp)->name->values, (*dp)->name->count) == 0)
			{
				NCadvise(NC_ENAMEINUSE, "variable \"%s\" in use with index %d",
					(*dp)->name->values, ii) ;
				return(-1) ;
			}
		}
		var[0] = NC_new_var(name,type,ndims,dims) ;
		if(var[0] == NULL)
			return(-1) ;
		if( NC_incr_array(handle->vars, (Void *)var) == NULL)
			return(-1) ;
	}
	if( NC_var_shape(*var, handle->dims) != -1)
	{
		return(handle->vars->count -1) ;
	}
		/* unwind */
	handle->vars->count-- ;
	NC_free_var(var[0]) ;
	return(-1) ;
}


/*
 * Recompute the shapes of all variables
 * Sets handle->begin_rec to start of first record variable
 * returns -1 on error
 */
NC_computeshapes( handle )
NC *handle ;
{
	NC_var **vbase, **vpp ;
	NC_var *first = NULL ;

	handle->begin_rec = 0 ;
	handle->recsize = 0 ;

	if(handle->vars == NULL)
		return(0) ;
	vbase = (NC_var **)handle->vars->values ;
	for( vpp =  vbase ;
		vpp < &vbase[handle->vars->count] ;
		vpp ++)
	{
		if( NC_var_shape(*vpp, handle->dims) == -1)
			return(-1) ;
	  	if(IS_RECVAR(*vpp))	
		{
	  		if(first == NULL)	
				first = *vpp ;
			handle->recsize += (*vpp)->len ;
		}
	}
	if(first != NULL)
	{
		handle->begin_rec = first->begin ;
		/*
	 	 * for special case of exactly one record variable, pack values
	 	 */
		if(handle->recsize == first->len)
			handle->recsize = *first->dsizes ;
	}
	return(handle->vars->count) ;
}


ncvarid( cdfid, name)
int cdfid ;
char *name ;
{
	NC *handle ;
	extern NC *NC_check_id() ;
	NC_var **dp ;
	int ii ;
	int len ;

	cdf_routine_name = "ncvarid" ;

	handle = NC_check_id(cdfid) ;
	if(handle == NULL)
		return(-1) ;
	if(handle->vars == NULL)
		return(-1) ;
	len = strlen(name) ;
	dp = (NC_var**)handle->vars->values ;
	for(ii = 0 ; ii < handle->vars->count ; ii++, dp++)
	{
		if( len == (*dp)->name->count &&
			strncmp(name, (*dp)->name->values, (*dp)->name->count) == 0)
		{
			return(ii) ;
		}
	}
	NCadvise(NC_ENOTVAR, "variable \"%s\" not found", name) ;
	return(-1) ;
}


/*
 * Given valid handle and varid, return var
 *  else NULL on error
 */
NC_var *
NC_hlookupvar( handle, varid )
NC *handle ;
int varid ;
{
	NC_array **ap ;

	if(varid == NC_GLOBAL) /* Global is error in this context */
	{
		return(NULL) ;
	}else if(handle->vars != NULL && varid >= 0 && varid < handle->vars->count)
	{
		ap = (NC_array **)handle->vars->values ;
		ap += varid ;
	} else {
		NCadvise(NC_ENOTVAR, "%d is not a valid variable id", varid) ;
		return( NULL ) ;
	}
	return((NC_var *)*ap) ;
}


/*
 * Given cdfid and varid, return var
 *  else NULL on error
 */
static NC_var *
NC_lookupvar( cdfid, varid )
int cdfid ;
int varid ;
{
	NC *handle ;
	extern NC *NC_check_id() ;

	handle = NC_check_id(cdfid) ;
	if(handle == NULL)
		return(NULL) ;

	return(NC_hlookupvar(handle,varid)) ;
}


ncvarinq( cdfid, varid, name, typep, ndimsp, dims, nattrsp)  
int cdfid ;
int varid ;
char *name ;
nc_type *typep ;
int *ndimsp ;
int dims[] ;
int *nattrsp ;
{
	NC_var *vp ;
	int ii ;

	cdf_routine_name = "ncvarinq" ;

	vp = NC_lookupvar( cdfid, varid ) ;
	if(vp == NULL)
		return(-1) ;

	if(name != NULL)
	{
		(void)strncpy( name, vp->name->values, vp->name->count) ;
		name[vp->name->count] = 0 ;
	}

	*typep = vp->type ;
	*ndimsp = vp->assoc->count ;
	for(ii = 0 ; ii < *ndimsp ; ii++)
	{
		dims[ii] = vp->assoc->values[ii] ;
	}
	if( vp->attrs != NULL)
	{
		*nattrsp = vp->attrs->count ;
	} else {
		*nattrsp = 0 ;
	}

	return(varid) ;
}


ncvarrename(cdfid, varid, newname)
int cdfid ;
int varid ; 
char *newname ;
{
	
	NC *handle ;
	NC_var **vpp ;
	int ii ;
	int len ;
	NC_string *old, *new ;
	extern NC *NC_check_id() ;
	extern NC_string *NC_new_string() ;
	extern NC_string *NC_re_string() ;
	extern void NC_free_string() ;

	cdf_routine_name = "ncvarrename" ;

	handle = NC_check_id(cdfid) ;
	if(handle == NULL)
		return(-1) ;
	if(!(handle->flags & NC_RDWR))
		return(-1) ;

	/* check for name in use */
	len = strlen(newname) ;
	vpp = (NC_var**)handle->vars->values ;
	for(ii = 0 ; ii < handle->vars->count ; ii++, vpp++)
	{
		if( len == (*vpp)->name->count &&
			strncmp(newname, (*vpp)->name->values, (*vpp)->name->count) == 0)
		{
			NCadvise(NC_ENAMEINUSE, "variable name \"%s\" in use with index %d",
				(*vpp)->name->values, ii) ;
			return(-1) ;
		}
	}

	if(varid == NC_GLOBAL) /* Global is error in this context */
	{
		NCadvise(NC_EGLOBAL, "action prohibited on NC_GLOBAL varid") ;
		return(-1) ;
	}else if(handle->vars != NULL && varid >= 0 && varid < handle->vars->count)
	{
		vpp = (NC_var **)handle->vars->values ;
		vpp += varid ;
	} else {
		NCadvise(NC_ENOTVAR, "%d is not a valid variable id", varid) ;
		return( -1 ) ;
	}

	old = (*vpp)->name ;
	if( NC_indefine(cdfid,TRUE) )
	{
		new = NC_new_string((unsigned)strlen(newname),newname) ;
		if( new == NULL)
			return(-1) ;
		(*vpp)->name = new ;
		NC_free_string(old) ;
		return(varid) ;
	} /* else */
	new = NC_re_string(old, (unsigned)strlen(newname),newname) ;
	if( new == NULL)
		return(-1) ;
	if(handle->flags & NC_HSYNC)
	{
		handle->xdrs->x_op = XDR_ENCODE ;
		if(!xdr_cdf(handle->xdrs, &handle) )
			return(-1) ;
		handle->flags &= ~(NC_NDIRTY | NC_HDIRTY) ;
	} else
		handle->flags |= NC_HDIRTY ;
	return(varid) ;
}


bool_t
xdr_NC_var(xdrs, vpp)
	XDR *xdrs;
	NC_var **vpp;
{
	u_long begin ;
	extern Size_t NC_typelen() ;

	if( xdrs->x_op == XDR_FREE)
	{
		NC_free_var((*vpp)) ;
		return(TRUE) ;
	}

	if( xdrs->x_op == XDR_DECODE )
	{
		*vpp = (NC_var *)m_alloc(sizeof(NC_var)) ;
		if( *vpp == NULL )
		{
			nc_serror("xdr_NC_var") ;
			return(FALSE) ;
		}
	}

	if( !xdr_NC_string(xdrs, &((*vpp)->name)))
		return(FALSE) ;
	if( !xdr_NC_iarray(xdrs, &((*vpp)->assoc)))
		return(FALSE) ;
	if( !xdr_NC_array(xdrs, &((*vpp)->attrs)))
		return(FALSE) ;

#ifdef USE_ENUM
	if (! xdr_enum(xdrs, &((*vpp)->type)) ) {
		return (FALSE);
	}
#else
	if (! xdr_int(xdrs, &((*vpp)->type)) ) {
		return (FALSE);
	}
#endif
	if (! xdr_u_long(xdrs, &((*vpp)->len)) ) {
		return (FALSE);
	}

	if( xdrs->x_op == XDR_DECODE )
		(*vpp)->szof = NC_typelen((*vpp)->type) ;

	if( xdrs->x_op == XDR_ENCODE )
 		begin = (*vpp)->begin ;
	if (! xdr_u_long(xdrs, &begin))
		return (FALSE);
	if( xdrs->x_op == XDR_DECODE )
		(*vpp)->begin = begin ;

	return( TRUE ) ;
}


/*
 * How much space will the xdr'd var take.
 *
 */
NC_xlen_var(vpp)
NC_var **vpp ;
{
	int len ;

	if(*vpp == NULL)
		return(4) ;

	len = NC_xlen_string((*vpp)->name) ;
	len += NC_xlen_iarray((*vpp)->assoc) ;
	len += NC_xlen_array((*vpp)->attrs) ;
	len += 12 ;

	return(len) ;
}
